#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.algeria import Algeria
from tests.common import CommonCountryTests


class TestAlgeria(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Algeria)

    def test_new_years_day(self):
        self.assertHolidayName(
            "رأس السنة الميلادية", (f"{year}-01-01" for year in self.full_range)
        )

    def test_amazigh_new_year(self):
        name = "رأس السنة الأمازيغية"
        self.assertHolidayName(name, (f"{year}-01-12" for year in range(2018, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 2018))

    def test_labor_day(self):
        self.assertHolidayName("عيد العمال", (f"{year}-05-01" for year in self.full_range))

    def test_independence_day(self):
        name_1964 = "عيد الاستقلال وجبهة التحرير الوطني"
        name_2005 = "عيد الاستقلال"
        self.assertHolidayName(
            name_1964, (f"{year}-07-05" for year in range(self.start_year, 2005))
        )
        self.assertHolidayName(name_2005, (f"{year}-07-05" for year in range(2005, self.end_year)))
        self.assertNoHolidayName(name_1964, range(2005, self.end_year))
        self.assertNoHolidayName(name_2005, range(self.start_year, 2005))

    def test_revolution_day(self):
        self.assertHolidayName("عيد الثورة", (f"{year}-11-01" for year in self.full_range))

    def test_islamic_new_year(self):
        name = "رأس السنة الهجرية"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-20",
            "2021-08-09",
            "2022-07-30",
            "2023-07-19",
            "2024-07-07",
            "2025-06-26",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_ashura(self):
        name = "عاشورة"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-08-29",
            "2021-08-18",
            "2022-08-08",
            "2023-07-28",
            "2024-07-16",
            "2025-07-05",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_prophets_birthday(self):
        name = "عيد المولد النبوي"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-10-29",
            "2021-10-18",
            "2022-10-08",
            "2023-09-27",
            "2024-09-15",
            "2025-09-04",
        )
        self.assertIslamicNoEstimatedHolidayName(name, self.full_range)

    def test_eid_al_fitr(self):
        name = "عيد الفطر"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-05-24",
            "2020-05-25",
            "2021-05-13",
            "2021-05-14",
            "2022-05-02",
            "2022-05-03",
            "2023-04-21",
            "2023-04-22",
            "2024-04-10",
            "2024-04-11",
            "2024-04-12",
            "2025-03-30",
            "2025-03-31",
            "2025-04-01",
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, range(1964, 1968), range(1969, 2000), range(2001, 2024)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, range(2024, 2033), range(2034, self.end_year)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 1968, 2000)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, 2033)

    def test_eid_al_adha(self):
        name = "عيد الأضحى"
        self.assertIslamicNoEstimatedHolidayName(
            name,
            "2020-07-31",
            "2020-08-01",
            "2021-07-20",
            "2021-07-21",
            "2022-07-09",
            "2022-07-10",
            "2023-06-28",
            "2023-06-29",
            "2023-06-30",
            "2024-06-16",
            "2024-06-17",
            "2024-06-18",
            "2025-06-06",
            "2025-06-07",
            "2025-06-08",
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 1, range(1964, 1969))
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 2, range(1969, 1974), range(1975, 2006), range(2008, 2023)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(
            name, 3, range(2006, 2008), range(2023, 2039), range(2040, self.end_year)
        )
        self.assertIslamicNoEstimatedHolidayNameCount(name, 4, 1974)
        self.assertIslamicNoEstimatedHolidayNameCount(name, 6, 2039)

    def test_easter_monday(self):
        name = "إثنين الفصح"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertChristianHolidayName(name, self.full_range)

    def test_ascension_day(self):
        name = "عيد الصعود"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertChristianHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "إثنين العنصرة"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertChristianHolidayName(name, self.full_range)

    def test_assumption_day(self):
        name = "عيد انتقال السيدة العذراء"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(name, (f"{year}-08-15" for year in self.full_range))

    def test_christmas_day(self):
        name = "عيد الميلاد"
        self.assertNoHolidayName(name)
        self.assertChristianHolidayName(name, (f"{year}-12-25" for year in self.full_range))

    def test_rosh_hashanah(self):
        name = "رأس السنة العبرية"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2020-09-19",
            "2021-09-07",
            "2022-09-26",
            "2023-09-16",
            "2024-10-03",
            "2025-09-23",
        )
        self.assertHebrewHolidayName(name, self.full_range)

    def test_yom_kippur(self):
        name = "يوم الغفران"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2020-09-28",
            "2021-09-16",
            "2022-10-05",
            "2023-09-25",
            "2024-10-12",
            "2025-10-02",
        )
        self.assertHebrewHolidayName(name, self.full_range)

    def test_pesach(self):
        name = "عيد الفصح اليهودي"
        self.assertNoHolidayName(name)
        self.assertHebrewHolidayName(
            name,
            "2020-04-09",
            "2021-03-28",
            "2022-04-16",
            "2023-04-06",
            "2024-04-23",
            "2025-04-13",
        )
        self.assertHebrewHolidayName(name, self.full_range)

    def test_weekend(self):
        for dt in (
            "1975-01-04",  # SAT.
            "1975-01-05",  # SUN.
            "1976-01-01",  # THU.
            "1976-01-02",  # FRI.
            "2008-01-03",  # THU.
            "2008-01-04",  # FRI.
            "2009-01-02",  # FRI.
            "2009-01-03",  # SAT.
        ):
            self.assertTrue(self.holidays.is_weekend(dt))

        for dt in (
            "1975-01-02",  # THU.
            "1975-01-03",  # FRI.
            "1976-01-03",  # SAT.
            "1976-01-04",  # SUN.
            "2008-01-05",  # SAT.
            "2008-01-06",  # SUN.
            "2009-01-01",  # THU.
            "2009-01-04",  # SUN.
        ):
            self.assertFalse(self.holidays.is_weekend(dt))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "رأس السنة الميلادية"),
            ("2022-01-12", "رأس السنة الأمازيغية"),
            ("2022-04-16", "عيد الفصح اليهودي"),
            ("2022-04-18", "إثنين الفصح"),
            ("2022-05-01", "عيد العمال"),
            ("2022-05-02", "عيد الفطر (تقديري)"),
            ("2022-05-03", "عيد الفطر (تقديري)"),
            ("2022-05-26", "عيد الصعود"),
            ("2022-06-06", "إثنين العنصرة"),
            ("2022-07-05", "عيد الاستقلال"),
            ("2022-07-09", "عيد الأضحى (تقديري)"),
            ("2022-07-10", "عيد الأضحى (تقديري)"),
            ("2022-07-30", "رأس السنة الهجرية (تقديري)"),
            ("2022-08-08", "عاشورة (تقديري)"),
            ("2022-08-15", "عيد انتقال السيدة العذراء"),
            ("2022-09-26", "رأس السنة العبرية"),
            ("2022-10-05", "يوم الغفران"),
            ("2022-10-08", "عيد المولد النبوي (تقديري)"),
            ("2022-11-01", "عيد الثورة"),
            ("2022-12-25", "عيد الميلاد"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-12", "Amazigh New Year"),
            ("2022-04-16", "Pesach"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-02", "Eid al-Fitr (estimated)"),
            ("2022-05-03", "Eid al-Fitr (estimated)"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-07-05", "Independence Day"),
            ("2022-07-09", "Eid al-Adha (estimated)"),
            ("2022-07-10", "Eid al-Adha (estimated)"),
            ("2022-07-30", "Islamic New Year (estimated)"),
            ("2022-08-08", "Ashura (estimated)"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-26", "Rosh Hashanah"),
            ("2022-10-05", "Yom Kippur"),
            ("2022-10-08", "Prophet's Birthday (estimated)"),
            ("2022-11-01", "Revolution Day"),
            ("2022-12-25", "Christmas Day"),
        )

    def test_l10n_fr(self):
        self.assertLocalizedHolidays(
            "fr",
            ("2022-01-01", "Jour de l'An Grégorien"),
            ("2022-01-12", "Jour de l'An Amazigh"),
            ("2022-04-16", "Pisah"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-02", "Aïd el-Fitr (estimé)"),
            ("2022-05-03", "Aïd el-Fitr (estimé)"),
            ("2022-05-26", "Ascension"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-07-05", "Fête de l'Indépendance"),
            ("2022-07-09", "Aïd el-Adha (estimé)"),
            ("2022-07-10", "Aïd el-Adha (estimé)"),
            ("2022-07-30", "Awal Moharram (estimé)"),
            ("2022-08-08", "Achoura (estimé)"),
            ("2022-08-15", "Assomption"),
            ("2022-09-26", "Roch Achana"),
            ("2022-10-05", "Youm Kippour"),
            ("2022-10-08", "El-Mawlid Ennabawi Echarif (estimé)"),
            ("2022-11-01", "Fête de la Révolution"),
            ("2022-12-25", "Noël"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from importlib.metadata import metadata
from unittest import TestCase

import holidays


class TestPackage(TestCase):
    def test_metadata(self):
        ph_metadata = metadata("holidays")

        for attr_name, attr_value in {
            "name": "holidays",
            "summary": "Open World Holidays Framework",
            "version": holidays.__version__,
        }.items():
            with self.subTest(attr=attr_name):
                self.assertIn(attr_name, ph_metadata)
                self.assertEqual(
                    ph_metadata[attr_name],
                    attr_value,
                    msg="You may need to run `make package` to update the metadata."
                    if attr_name == "version"
                    else None,
                )

        for attr_name in (
            "classifier",
            "description",
            "keywords",
            "license-expression",
            "license-file",
            "maintainer",
            "project-url",
            "requires-python",
        ):
            self.assertIn(attr_name, ph_metadata)
            self.assertTrue(ph_metadata[attr_name], attr_name)

            if attr_name == "maintainer":
                for maintainer in ("Arkadii Yakovets", "Panpakorn Siripanich", "Serhii Murza"):
                    self.assertIn(maintainer, ph_metadata["maintainer"])



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.andorra import Andorra
from tests.common import CommonCountryTests


class TestAndorra(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Andorra)

    def test_new_years_day(self):
        self.assertHolidayName("Cap d'Any", (f"{year}-01-01" for year in self.full_range))

    def test_epiphany(self):
        self.assertHolidayName("Reis", (f"{year}-01-06" for year in self.full_range))

    def test_carnival(self):
        name = "Carnaval"
        self.assertHolidayName(
            name,
            "2020-02-24",
            "2021-02-15",
            "2022-02-28",
            "2023-02-20",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayName(name, self.full_range)

    def test_constitution_day(self):
        name = "Dia de la Constitució"
        self.assertHolidayName(name, (f"{year}-03-14" for year in range(1994, self.end_year)))
        self.assertNoHolidayName(name, range(self.start_year, 1994))

    def test_good_friday(self):
        name = "Divendres Sant"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Dilluns de Pasqua"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_labor_day(self):
        self.assertHolidayName("Festa del treball", (f"{year}-05-01" for year in self.full_range))

    def test_whit_monday(self):
        name = "Dilluns de Pentecosta"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_assumption_day(self):
        self.assertHolidayName("Assumpció", (f"{year}-08-15" for year in self.full_range))

    def test_our_lady_of_meritxell(self):
        self.assertHolidayName(
            "Nostra Senyora de Meritxell", (f"{year}-09-08" for year in self.full_range)
        )

    def test_all_saints_day(self):
        self.assertHolidayName("Tots Sants", (f"{year}-11-01" for year in self.full_range))

    def test_immaculate_conception(self):
        self.assertHolidayName(
            "Immaculada Concepció", (f"{year}-12-08" for year in self.full_range)
        )

    def test_christmas_day(self):
        self.assertHolidayName("Nadal", (f"{year}-12-25" for year in self.full_range))

    def test_saint_stephens_day(self):
        self.assertHolidayName("Sant Esteve", (f"{year}-12-26" for year in self.full_range))

    def test_saint_rochs_day(self):
        name = "Sant Roc"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "02":
                self.assertHolidayName(
                    name, holidays, (f"{year}-08-16" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_anthonys_day(self):
        name = "Sant Antoni"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "04":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-17" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_peters_day(self):
        name = "Sant Pere"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "05":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-29" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_julians_day(self):
        name = "Sant Julià"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "06":
                self.assertHolidayName(
                    name, holidays, (f"{year}-01-07" for year in self.full_range)
                )
            else:
                self.assertNoHolidayName(name, holidays)

    def test_virgin_mary_of_canolich(self):
        name = "Diada de Canòlich"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "06":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-05-30",
                    "2021-05-29",
                    "2022-05-28",
                    "2023-05-27",
                    "2024-05-25",
                    "2025-05-31",
                )
                self.assertHolidayName(name, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_sant_julia_de_loria_festival(self):
        name = "Festa Major de Sant Julià de Lòria"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "06":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-07-27",
                    "2020-07-28",
                    "2021-08-02",
                    "2021-08-03",
                    "2022-08-01",
                    "2022-08-02",
                    "2023-07-31",
                    "2023-08-01",
                    "2024-07-29",
                    "2024-07-30",
                    "2025-07-28",
                    "2025-07-29",
                )
                self.assertHolidayNameCount(name, 2, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_andorra_la_vella_festival(self):
        name = "Festa Major d'Andorra la Vella"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "07":
                self.assertHolidayName(
                    name,
                    holidays,
                    "2020-08-01",
                    "2020-08-02",
                    "2020-08-03",
                    "2021-08-07",
                    "2021-08-08",
                    "2021-08-09",
                    "2022-08-06",
                    "2022-08-07",
                    "2022-08-08",
                    "2023-08-05",
                    "2023-08-06",
                    "2023-08-07",
                    "2024-08-03",
                    "2024-08-04",
                    "2024-08-05",
                    "2025-08-02",
                    "2025-08-03",
                    "2025-08-04",
                )
                self.assertHolidayNameCount(name, 3, holidays, self.full_range)
            else:
                self.assertNoHolidayName(name, holidays)

    def test_saint_michael_of_engolasters_day(self):
        name = "Sant Miquel d'Engolasters"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "08":
                self.assertHolidayName(
                    name, holidays, (f"{year}-05-07" for year in range(1979, self.end_year))
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1979))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_parish_foundation_day(self):
        name = "Diada de la creació de la parròquia"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "08":
                self.assertHolidayName(
                    name, holidays, (f"{year}-06-14" for year in range(1979, 1997))
                )
                self.assertHolidayName(
                    name,
                    holidays,
                    "1997-06-15",
                    "1998-06-14",
                    "1999-06-20",
                    "2000-06-18",
                    "2020-06-14",
                    "2021-06-20",
                    "2022-06-19",
                    "2023-06-18",
                    "2024-06-16",
                    "2025-06-15",
                )
                self.assertHolidayName(name, holidays, range(1979, self.end_year))
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1979))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_escaldes_engordany_festival(self):
        name = "Festa Major d'Escaldes-Engordany"
        self.assertNoHolidayName(name)
        for subdiv, holidays in self.subdiv_holidays.items():
            if subdiv == "08":
                self.assertHolidayName(
                    name,
                    holidays,
                    (f"{year}-07-25" for year in range(1979, self.end_year)),
                    (f"{year}-07-26" for year in range(1979, self.end_year)),
                )
                self.assertNoHolidayName(name, holidays, range(self.start_year, 1979))
            else:
                self.assertNoHolidayName(name, holidays)

    def test_epiphany_eve(self):
        name = "Vigília de Reis (a partir de les 13h)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-01-05" for year in self.full_range))

    def test_maundy_thursday(self):
        name = "Dijous Sant (a partir de les 13h)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(
            name,
            "2020-04-09",
            "2021-04-01",
            "2022-04-14",
            "2023-04-06",
            "2024-03-28",
            "2025-04-17",
        )
        self.assertGovernmentHolidayName(name, self.full_range)

    def test_christmas_eve(self):
        name = "Vigília de Nadal (a partir de les 13h)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-12-24" for year in self.full_range))

    def test_new_years_eve(self):
        name = "Vigília de Cap d'Any (a partir de les 13h)"
        self.assertNoHolidayName(name)
        self.assertGovernmentHolidayName(name, (f"{year}-12-31" for year in self.full_range))

    def test_2023(self):
        self.assertHolidaysInYear(
            2023,
            ("2023-01-01", "Cap d'Any"),
            ("2023-01-06", "Reis"),
            ("2023-02-20", "Carnaval"),
            ("2023-03-14", "Dia de la Constitució"),
            ("2023-04-07", "Divendres Sant"),
            ("2023-04-10", "Dilluns de Pasqua"),
            ("2023-05-01", "Festa del treball"),
            ("2023-05-29", "Dilluns de Pentecosta"),
            ("2023-08-15", "Assumpció"),
            ("2023-09-08", "Nostra Senyora de Meritxell"),
            ("2023-11-01", "Tots Sants"),
            ("2023-12-08", "Immaculada Concepció"),
            ("2023-12-25", "Nadal"),
            ("2023-12-26", "Sant Esteve"),
        )

    def test_government_2023(self):
        self.assertGovernmentHolidaysInYear(
            2023,
            ("2023-01-05", "Vigília de Reis (a partir de les 13h)"),
            ("2023-04-06", "Dijous Sant (a partir de les 13h)"),
            ("2023-12-24", "Vigília de Nadal (a partir de les 13h)"),
            ("2023-12-31", "Vigília de Cap d'Any (a partir de les 13h)"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Cap d'Any"),
            ("2024-01-05", "Vigília de Reis (a partir de les 13h)"),
            ("2024-01-06", "Reis"),
            ("2024-01-07", "Sant Julià"),
            ("2024-01-17", "Sant Antoni"),
            ("2024-02-12", "Carnaval"),
            ("2024-03-14", "Dia de la Constitució"),
            ("2024-03-28", "Dijous Sant (a partir de les 13h)"),
            ("2024-03-29", "Divendres Sant"),
            ("2024-04-01", "Dilluns de Pasqua"),
            ("2024-05-01", "Festa del treball"),
            ("2024-05-07", "Sant Miquel d'Engolasters"),
            ("2024-05-20", "Dilluns de Pentecosta"),
            ("2024-05-25", "Diada de Canòlich"),
            ("2024-06-16", "Diada de la creació de la parròquia"),
            ("2024-06-29", "Sant Pere"),
            ("2024-07-25", "Festa Major d'Escaldes-Engordany"),
            ("2024-07-26", "Festa Major d'Escaldes-Engordany"),
            ("2024-07-29", "Festa Major de Sant Julià de Lòria"),
            ("2024-07-30", "Festa Major de Sant Julià de Lòria"),
            ("2024-08-03", "Festa Major d'Andorra la Vella"),
            ("2024-08-04", "Festa Major d'Andorra la Vella"),
            ("2024-08-05", "Festa Major d'Andorra la Vella"),
            ("2024-08-15", "Assumpció"),
            ("2024-08-16", "Sant Roc"),
            ("2024-09-08", "Nostra Senyora de Meritxell"),
            ("2024-11-01", "Tots Sants"),
            ("2024-12-08", "Immaculada Concepció"),
            ("2024-12-24", "Vigília de Nadal (a partir de les 13h)"),
            ("2024-12-25", "Nadal"),
            ("2024-12-26", "Sant Esteve"),
            ("2024-12-31", "Vigília de Cap d'Any (a partir de les 13h)"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-05", "Epiphany Eve (from 1pm)"),
            ("2024-01-06", "Epiphany"),
            ("2024-01-07", "Saint Julian's Day"),
            ("2024-01-17", "Saint Anthony's Day"),
            ("2024-02-12", "Carnival"),
            ("2024-03-14", "Constitution Day"),
            ("2024-03-28", "Maundy Thursday (from 1pm)"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-05-01", "Labor Day"),
            ("2024-05-07", "Saint Michael of Engolasters' Day"),
            ("2024-05-20", "Whit Monday"),
            ("2024-05-25", "Virgin Mary of Canòlich"),
            ("2024-06-16", "Parish foundation day"),
            ("2024-06-29", "Saint Peter's Day"),
            ("2024-07-25", "Escaldes-Engordany Festival"),
            ("2024-07-26", "Escaldes-Engordany Festival"),
            ("2024-07-29", "Sant Julià de Lòria Festival"),
            ("2024-07-30", "Sant Julià de Lòria Festival"),
            ("2024-08-03", "Andorra la Vella Festival"),
            ("2024-08-04", "Andorra la Vella Festival"),
            ("2024-08-05", "Andorra la Vella Festival"),
            ("2024-08-15", "Assumption Day"),
            ("2024-08-16", "Saint Roch's Day"),
            ("2024-09-08", "Our Lady of Meritxell"),
            ("2024-11-01", "All Saints' Day"),
            ("2024-12-08", "Immaculate Conception"),
            ("2024-12-24", "Christmas Eve (from 1pm)"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Saint Stephen's Day"),
            ("2024-12-31", "New Year's Eve (from 1pm)"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2024-01-01", "Новий рік"),
            ("2024-01-05", "Переддень Богоявлення (з 13:00)"),
            ("2024-01-06", "Богоявлення"),
            ("2024-01-07", "День Святого Юліана"),
            ("2024-01-17", "День Святого Антонія"),
            ("2024-02-12", "Карнавал"),
            ("2024-03-14", "День Конституції"),
            ("2024-03-28", "Великий четвер (з 13:00)"),
            ("2024-03-29", "Страсна пʼятниця"),
            ("2024-04-01", "Великодній понеділок"),
            ("2024-05-01", "День праці"),
            ("2024-05-07", "День Святого Михаїла Енголастерського"),
            ("2024-05-20", "День Святого Духа"),
            ("2024-05-25", "День Богоматері Каноліхської"),
            ("2024-06-16", "День створення парафії"),
            ("2024-06-29", "День Святого Петра"),
            ("2024-07-25", "Свято парафії Ескальдес-Енгордань"),
            ("2024-07-26", "Свято парафії Ескальдес-Енгордань"),
            ("2024-07-29", "Свято парафії Сант-Жулія-де-Лорія"),
            ("2024-07-30", "Свято парафії Сант-Жулія-де-Лорія"),
            ("2024-08-03", "Свято парафії Андорра-ла-Велья"),
            ("2024-08-04", "Свято парафії Андорра-ла-Велья"),
            ("2024-08-05", "Свято парафії Андорра-ла-Велья"),
            ("2024-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2024-08-16", "День Святого Роха"),
            ("2024-09-08", "День Богоматері Мерічелльської"),
            ("2024-11-01", "День усіх святих"),
            ("2024-12-08", "Непорочне зачаття Діви Марії"),
            ("2024-12-24", "Святий вечір (з 13:00)"),
            ("2024-12-25", "Різдво Христове"),
            ("2024-12-26", "День Святого Стефана"),
            ("2024-12-31", "Переддень Нового року (з 13:00)"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import unittest
import warnings
from collections import defaultdict
from collections.abc import Callable
from concurrent.futures import ProcessPoolExecutor
from datetime import date
from pathlib import Path
from unittest import mock

import pytest

import holidays
from holidays.calendars.gregorian import FRI, SAT
from holidays.holiday_base import HolidayBase
from holidays.utils import (
    country_holidays,
    CountryHoliday,
    financial_holidays,
    list_localized_countries,
    list_localized_financial,
    list_long_breaks,
    list_supported_countries,
    list_supported_financial,
)
from tests.common import PYTHON_LATEST_SUPPORTED_VERSION, PYTHON_VERSION


class TestCountryHolidays(unittest.TestCase):
    def setUp(self):
        self.holidays = country_holidays("US")

    def test_country(self):
        self.assertEqual(self.holidays.country, "US")

    def test_country_single_year(self):
        test_holidays = country_holidays("US", years=2021)
        self.assertEqual(test_holidays.years, {2021})

    def test_country_multiple_years(self):
        test_holidays = country_holidays("US", years=(2015, 2021))
        self.assertEqual(test_holidays.years, {2015, 2021})

    def test_country_range_years(self):
        test_holidays = country_holidays("US", years=range(2010, 2015))
        self.assertEqual(test_holidays.years, set(range(2010, 2015)))

    def test_country_subdivision(self):
        test_holidays = country_holidays("US", subdiv="NY")
        self.assertEqual(test_holidays.subdiv, "NY")

    def test_invalid_country_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            country_holidays("XXXX")

    def test_invalid_country_subdivision_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            country_holidays("US", subdiv="XXXX")

    def test_country_holiday_class_deprecation(self):
        with self.assertWarns(DeprecationWarning) as ctx:
            CountryHoliday("IT")
        self.assertIn("CountryHoliday is deprecated", str(ctx.warning))


class TestFinancialHolidays(unittest.TestCase):
    def setUp(self):
        self.holidays = financial_holidays("XNYS")

    def test_market(self):
        self.assertEqual(self.holidays.market, "XNYS")

    def test_market_single_year(self):
        test_holidays = financial_holidays("XNYS", years=2021)
        self.assertEqual(test_holidays.years, {2021})

    def test_market_multiple_years(self):
        test_holidays = financial_holidays("XNYS", years=(2015, 2021))
        self.assertEqual(test_holidays.years, {2015, 2021})

    def test_market_range_years(self):
        test_holidays = financial_holidays("XNYS", years=range(2010, 2015))
        self.assertEqual(test_holidays.years, set(range(2010, 2015)))

    def test_invalid_market_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            financial_holidays("XXXX")

    def test_invalid_market_subdivision_raises_not_implemented(self):
        with self.assertRaises(NotImplementedError):
            financial_holidays("XNYS", subdiv="XXXX")


class TestAllInSameYear(unittest.TestCase):
    """Ensure only holidays in the year(s) requested are returned."""

    years = set(range(1950, 2051))

    @staticmethod
    def _check_single_entity_worker(
        args: tuple[str, Callable[..., HolidayBase], set[int]],
    ) -> None:
        entity, entity_func, years = args
        warnings.filterwarnings("ignore", category=DeprecationWarning)

        # Check each year individually.
        for year in years:
            for dt in entity_func(entity, years=year):
                if not isinstance(dt, date):
                    raise AssertionError(f"{entity}: {dt} is not a date instance")
                if dt.year != year:
                    raise AssertionError(f"{entity}: date {dt} not in year {year}")

        # Check full range at once.
        all_holidays = entity_func(entity, years=years)
        if all_holidays.years != years:
            raise AssertionError(f"{entity}: years mismatch {all_holidays.years} != {years}")

    def _check_holidays_years(
        self, entity_func: Callable[..., HolidayBase], entity_list: dict[str, list[str]]
    ):
        """
        Only holidays within the requested years should be returned.
        This prevents triggering a `RuntimeError: dictionary changed size during iteration`.

        This is a logic test, not a compatibility test, so for expediency it is executed
        only once using the latest supported Python version.
        """
        with ProcessPoolExecutor() as executor:
            list(
                executor.map(
                    self._check_single_entity_worker,
                    [(entity, entity_func, self.years) for entity in entity_list],
                )
            )

    @pytest.mark.skipif(
        PYTHON_VERSION != PYTHON_LATEST_SUPPORTED_VERSION,
        reason="Run once on the latest Python version only",
    )
    @mock.patch("pathlib.Path.rglob", return_value=())
    def test_all_countries(self, _unused_mock):
        self._check_holidays_years(country_holidays, list_supported_countries())

    @pytest.mark.skipif(
        PYTHON_VERSION != PYTHON_LATEST_SUPPORTED_VERSION,
        reason="Run once on the latest Python version only",
    )
    @mock.patch("pathlib.Path.rglob", return_value=())
    def test_all_financial(self, _unused_mock):
        self._check_holidays_years(financial_holidays, list_supported_financial())


class TestListLocalizedEntities(unittest.TestCase):
    def assertLocalizedEntities(self, localized_entities, supported_entities):  # noqa: N802
        tests_dir = Path(__file__).parent
        locale_dir = tests_dir.parent / "holidays" / "locale"

        # Collect `<locale>` part from
        # holidays/locale/<locale>/LC_MESSAGES/<entity_code>.mo.
        entity_to_languages = defaultdict(list)
        for path in locale_dir.rglob("*.mo"):
            entity_to_languages[path.stem].append(path.parts[-3])

        for entity_code in supported_entities.keys():
            actual_languages = sorted(entity_to_languages.get(entity_code, []))
            expected_languages = localized_entities.get(entity_code, [])

            self.assertEqual(
                actual_languages,
                expected_languages,
                f"The supported languages for {entity_code} don't match its actual languages: "
                f"{set(actual_languages).difference(set(expected_languages))}",
            )

            entity = getattr(holidays, entity_code)
            self.assertEqual(
                list(entity.supported_languages),
                expected_languages,
                f"The supported languages for {entity_code} don't match "
                "its `supported_languages`.",
            )

            if expected_languages:
                self.assertIn(
                    entity.default_language,
                    expected_languages,
                    "The `default_language` must be listed in "
                    f"`supported_languages` for {entity_code}",
                )

    def test_localized_countries(self):
        self.assertLocalizedEntities(
            list_localized_countries(), list_supported_countries(include_aliases=False)
        )

    def test_localized_financial(self):
        self.assertLocalizedEntities(
            list_localized_financial(), list_supported_financial(include_aliases=False)
        )


class TestListSupportedEntities(unittest.TestCase):
    def test_list_supported_countries(self):
        supported_countries = list_supported_countries(include_aliases=False)

        for country in ("AR", "IM", "ZA"):
            self.assertIn(country, supported_countries)

        us_subdivisions = supported_countries["US"]
        self.assertIn("CA", us_subdivisions)
        self.assertIsInstance(us_subdivisions, list)

        countries_count = sum(
            1 for path in Path("holidays/countries").glob("*.py") if path.stem != "__init__"
        )
        self.assertEqual(countries_count, len(supported_countries))

    def test_list_supported_financial(self):
        supported_financial = list_supported_financial(include_aliases=False)

        for market in ("BVMF", "IFEU", "XECB", "XNYS"):
            self.assertIn(market, supported_financial)

        xnys_subdivisions = supported_financial.get("XNYS", [])
        self.assertIsInstance(xnys_subdivisions, list)

        financial_count = sum(
            1 for path in Path("holidays/financial").glob("*.py") if path.stem != "__init__"
        )
        self.assertEqual(financial_count, len(supported_financial))


class CountryStub1(HolidayBase):
    country = "CS1"

    def _populate(self, year: int) -> None:
        super()._populate(year)
        self._add_holiday_jul_4("Custom July 4th Holiday")
        self._add_holiday_mar_4("Custom March 4th Holiday")
        self._add_holiday_mar_5("Custom March 5th Holiday")
        self._add_holiday_mar_6("Custom March 6th Holiday")


class CountryStub2(HolidayBase):
    country = "CS2"

    def _populate(self, year: int) -> None:
        super()._populate(year)
        self._add_holiday_jul_4("Custom July 4th Holiday")
        self._add_holiday_dec_26("Custom December 26th Holiday")


class CountryStub3(HolidayBase):
    country = "CS3"
    weekend = {FRI, SAT}

    def _populate(self, year: int) -> None:
        super()._populate(year)
        self._add_holiday_apr_10("Custom April 10th Holiday")


class CountryStub4(HolidayBase):
    country = "CS4"

    def _populate(self, year: int) -> None:
        super()._populate(year)
        self._add_holiday_aug_11("Custom August 11th Holiday")
        self._add_holiday_aug_12("Custom August 12th Holiday")
        self._add_holiday_dec_5("Custom December 5th Holiday")


class CountryStub5(HolidayBase):
    country = "CS5"

    def _populate(self, year: int) -> None:
        super()._populate(year)

        if self._year == 2024:
            self._add_holiday_jan_1("Custom January 1st Holiday")
        if self._year == 2021:
            self._add_holiday_dec_31("Custom December 31st Holiday")


class TestListLongBreaks(unittest.TestCase):
    def assertLongBreaksEqual(  # noqa: N802
        self, instance, expected, *, minimum_break_length=3, require_weekend_overlap=True
    ):
        result = list_long_breaks(
            instance,
            minimum_break_length=minimum_break_length,
            require_weekend_overlap=require_weekend_overlap,
        )
        self.assertEqual(result, expected)

    def test_long_break_single(self):
        cs1 = CountryStub1(years=2025)
        self.assertLongBreaksEqual(
            cs1,
            [[date(2025, 7, 4), date(2025, 7, 5), date(2025, 7, 6)]],
        )

    def test_long_break_multiple(self):
        cs2 = CountryStub2(years=2025)
        self.assertLongBreaksEqual(
            cs2,
            [
                [date(2025, 7, 4), date(2025, 7, 5), date(2025, 7, 6)],
                [date(2025, 12, 26), date(2025, 12, 27), date(2025, 12, 28)],
            ],
        )

    def test_long_break_require_weekend_overlap(self):
        cs1 = CountryStub1(years=2025)
        self.assertLongBreaksEqual(
            cs1,
            [[date(2025, 7, 4), date(2025, 7, 5), date(2025, 7, 6)]],
        )
        self.assertLongBreaksEqual(
            cs1,
            [
                [date(2025, 3, 4), date(2025, 3, 5), date(2025, 3, 6)],
                [date(2025, 7, 4), date(2025, 7, 5), date(2025, 7, 6)],
            ],
            require_weekend_overlap=False,
        )

    def test_long_break_custom_weekend(self):
        cs3 = CountryStub3(years=2025)
        self.assertLongBreaksEqual(
            cs3,
            [[date(2025, 4, 10), date(2025, 4, 11), date(2025, 4, 12)]],
        )

    def test_long_break_no_holidays(self):
        self.assertLongBreaksEqual(HolidayBase(), [])

    def test_long_break_custom_minimum_length(self):
        cs4 = CountryStub4(years=2025)
        self.assertLongBreaksEqual(
            cs4,
            [[date(2025, 8, 9), date(2025, 8, 10), date(2025, 8, 11), date(2025, 8, 12)]],
            minimum_break_length=4,
        )
        self.assertLongBreaksEqual(
            cs4,
            [
                [date(2025, 8, 9), date(2025, 8, 10), date(2025, 8, 11), date(2025, 8, 12)],
                [date(2025, 12, 5), date(2025, 12, 6), date(2025, 12, 7)],
            ],
        )

    def test_long_break_across_years(self):
        cs5 = CountryStub5(years=2024)
        self.assertLongBreaksEqual(
            cs5,
            [[date(2023, 12, 30), date(2023, 12, 31), date(2024, 1, 1)]],
        )
        cs5 = CountryStub5(years=2021)
        self.assertLongBreaksEqual(
            cs5,
            [[date(2021, 12, 31), date(2022, 1, 1), date(2022, 1, 2)]],
        )

    def test_long_break_real_country_holidays_data(self):
        self.assertEqual(
            list_long_breaks(country_holidays("AU", subdiv="NSW", years=2024)),
            [
                [date(2023, 12, 30), date(2023, 12, 31), date(2024, 1, 1)],
                [date(2024, 1, 26), date(2024, 1, 27), date(2024, 1, 28)],
                [date(2024, 3, 29), date(2024, 3, 30), date(2024, 3, 31), date(2024, 4, 1)],
                [date(2024, 6, 8), date(2024, 6, 9), date(2024, 6, 10)],
                [date(2024, 10, 5), date(2024, 10, 6), date(2024, 10, 7)],
            ],
        )

    def test_long_break_real_financial_holidays_data(self):
        self.assertEqual(
            list_long_breaks(financial_holidays("XNSE", years=2024)),
            [
                [date(2024, 1, 26), date(2024, 1, 27), date(2024, 1, 28)],
                [date(2024, 3, 8), date(2024, 3, 9), date(2024, 3, 10)],
                [date(2024, 3, 23), date(2024, 3, 24), date(2024, 3, 25)],
                [date(2024, 3, 29), date(2024, 3, 30), date(2024, 3, 31)],
                [date(2024, 6, 15), date(2024, 6, 16), date(2024, 6, 17)],
                [date(2024, 11, 1), date(2024, 11, 2), date(2024, 11, 3)],
                [date(2024, 11, 15), date(2024, 11, 16), date(2024, 11, 17)],
            ],
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bermuda import Bermuda
from tests.common import CommonCountryTests


class TestBermuda(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bermuda)

    def test_special_holidays(self):
        for dt, name in (
            ("2007-06-05", "Public Holiday"),
            ("2019-11-04", "Portuguese Welcome 170th Anniversary"),
            ("2021-10-18", "Flora Duffy Day"),
            ("2023-05-08", "The Coronation of His Majesty King Charles III"),
        ):
            self.assertHolidayName(name, dt)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dts = (
            "2011-01-03",
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_bermuda_day(self):
        name = "Bermuda Day"
        self.assertHolidayName(name, (f"{year}-05-24" for year in range(self.start_year, 2018)))
        self.assertHolidayName(
            name,
            "2018-05-25",
            "2019-05-24",
            "2020-05-29",
            "2021-05-28",
            "2022-05-27",
            "2023-05-26",
            "2024-05-24",
            "2025-05-23",
        )
        self.assertHolidayName(name, range(2018, self.end_year))

        obs_dts = (
            "2008-05-26",
            "2009-05-25",
            "2014-05-26",
            "2015-05-25",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "1997-06-16",
            "1998-06-15",
            "1999-06-21",
            "2000-06-12",
            "2001-06-11",
            "2006-06-12",
            "2007-06-11",
            "2008-06-16",
        )
        self.assertHolidayName(name, range(self.start_year, 2009))
        self.assertNoHolidayName(name, range(2009, self.end_year))

    def test_national_heroes_day(self):
        name = "National Heroes Day"
        self.assertHolidayName(
            name,
            "2008-10-13",
            "2009-06-15",
            "2010-06-21",
            "2020-06-15",
            "2021-06-21",
            "2022-06-20",
            "2023-06-19",
            "2024-06-17",
            "2025-06-16",
        )
        self.assertHolidayName(name, range(2008, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2008))

    def test_cup_match_and_emancipation_day(self):
        name_1948 = "Cup Match Day"
        self.assertHolidayName(
            name_1948,
            "1995-08-03",
            "1996-08-01",
            "1997-07-31",
            "1998-07-30",
            "1999-07-29",
        )
        self.assertHolidayName(name_1948, range(self.start_year, 2000))
        self.assertNoHolidayName(name_1948, range(2000, self.end_year))

        name_2000 = "Emancipation Day"
        self.assertHolidayName(
            name_2000,
            "2021-07-29",
            "2022-07-28",
            "2023-08-03",
            "2024-08-01",
            "2025-07-31",
        )
        self.assertHolidayName(name_2000, range(2000, self.end_year))
        self.assertNoHolidayName(name_2000, range(self.start_year, 2000))

    def test_somers_and_mary_prince_day(self):
        name_1948 = "Somers Day"
        self.assertHolidayName(
            name_1948,
            "2016-07-29",
            "2017-08-04",
            "2018-08-03",
            "2019-08-02",
        )
        self.assertHolidayName(name_1948, range(self.start_year, 2020))
        self.assertNoHolidayName(name_1948, range(2020, self.end_year))

        name_2020 = "Mary Prince Day"
        self.assertHolidayName(
            name_2020,
            "2020-07-31",
            "2021-07-30",
            "2022-07-29",
            "2023-08-04",
            "2024-08-02",
            "2025-08-01",
        )
        self.assertHolidayName(name_2020, range(2020, self.end_year))
        self.assertNoHolidayName(name_2020, range(self.start_year, 2020))

    def test_labor_day(self):
        name = "Labour Day"
        self.assertHolidayName(
            name,
            "2020-09-07",
            "2021-09-06",
            "2022-09-05",
            "2023-09-04",
            "2024-09-02",
            "2025-09-01",
        )
        self.assertHolidayName(name, self.full_range)

    def test_remembrance_day(self):
        name = "Remembrance Day"
        self.assertHolidayName(name, (f"{year}-11-11" for year in self.full_range))

        obs_dts = (
            "2007-11-12",
            "2012-11-12",
            "2017-11-13",
            "2018-11-12",
            "2023-11-13",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dts = (
            "2010-12-27",
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dts = (
            "2009-12-28",
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dts)
        self.assertNoNonObservedHoliday(obs_dts)

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "New Year's Day"),
            ("2025-04-18", "Good Friday"),
            ("2025-05-23", "Bermuda Day"),
            ("2025-06-16", "National Heroes Day"),
            ("2025-07-31", "Emancipation Day"),
            ("2025-08-01", "Mary Prince Day"),
            ("2025-09-01", "Labour Day"),
            ("2025-11-11", "Remembrance Day"),
            ("2025-12-25", "Christmas Day"),
            ("2025-12-26", "Boxing Day"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-05-24", "Bermuda Day"),
            ("2024-06-17", "National Heroes Day"),
            ("2024-08-01", "Emancipation Day"),
            ("2024-08-02", "Mary Prince Day"),
            ("2024-09-02", "Labour Day"),
            ("2024-11-11", "Remembrance Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-03-29", "Good Friday"),
            ("2024-05-24", "Bermuda Day"),
            ("2024-06-17", "National Heroes Day"),
            ("2024-08-01", "Emancipation Day"),
            ("2024-08-02", "Mary Prince Day"),
            ("2024-09-02", "Labor Day"),
            ("2024-11-11", "Remembrance Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.calendars.gregorian import APR, MAY, AUG, SEP, _timedelta
from holidays.calendars.thai import KHMER_CALENDAR
from holidays.groups import InternationalHolidays, StaticHolidays, ThaiCalendarHolidays
from holidays.holiday_base import HolidayBase


class Cambodia(HolidayBase, InternationalHolidays, StaticHolidays, ThaiCalendarHolidays):
    """Cambodia holidays.

    References:
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Cambodia>
        * [Public Holidays (English)](https://web.archive.org/web/20241117220849/https://www.nbc.gov.kh/english/news_and_events/official_holiday.php)
        * [Public Holidays (Khmer)](https://web.archive.org/web/20250401111433/https://www.nbc.gov.kh/news_and_events/official_holiday.php)
        * [Sub-decree On Peace Day in Cambodia](https://web.archive.org/web/20250427180236/https://admin.taftac-cambodia.org/public/pdf_file/regulation_1704245695_Z7IALJjE.pdf)
        * <http://archive.today/2026.01.22-052218/https://www.khmertimeskh.com/501424903/24-public-holidays-for-2024-peace-day-now-included/>
        * <http://archive.today/2026.01.22-051938/https://www.khmertimeskh.com/501551204/govt-announces-22-public-holidays-for-next-year/>

    Checked with:
        * [ASEAN National Holidays 2022](https://web.archive.org/web/20250414071145/https://asean.org/wp-content/uploads/2021/12/ASEAN-National-Holidays-2022.pdf)
        * [ASEAN National Holidays 2023](https://web.archive.org/web/20250414071156/https://asean.org/wp-content/uploads/2022/12/ASEAN-Public-Holidays-2023.pdf)
        * <https://web.archive.org/web/20250416140353/https://www.timeanddate.com/holidays/cambodia/>

    Limitations:
        * Cambodian holidays only work from 1993 onwards.
        * Exact Public Holidays as per Cambodia's Official Gazette are only available
           from 2015 onwards.
        * Cambodian Lunar Calendar Holidays only work until 2157 (B.E. 2701) as we only
           have Thai year-type data for cross-checking until then.
    """

    country = "KH"
    default_language = "km"
    supported_languages = ("en_US", "km", "th")
    # Available post-Independence from 1993 afterwards
    start_year = 1993

    def __init__(self, *args, **kwargs):
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, cls=CambodiaStaticHolidays)
        ThaiCalendarHolidays.__init__(self, KHMER_CALENDAR)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # Fixed Holidays

        #  ទិវាចូលឆ្នាំសាកល
        # Status: In-Use.

        # International New Year Day.
        self._add_new_years_day(tr("ទិវាចូលឆ្នាំសាកល"))

        #  ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍
        # Status: In-Use.
        # Commemorates the end of the Khmer Rouge regime in 1979

        # Day of Victory over the Genocidal Regime.
        self._add_holiday_jan_7(tr("ទិវាជ័យជម្នះលើរបបប្រល័យពូជសាសន៍"))

        # ទិវាអន្តរជាតិនារី
        # Status: In-Use.

        # International Women's Rights Day.
        self._add_womens_day(tr("ទិវាអន្តរជាតិនារី"))

        #  ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ
        # Status: In-Use.
        # Usually falls on April 13th except for 2017-2018, 2021-2023, 2025-2027, 2029-2031
        # for years 2001-2050.

        if self._year != 2020:
            # Khmer New Year's Day.
            sangkranta = tr("ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ")
            sangkranta_years_apr_14 = {
                2017,
                2018,
                2021,
                2022,
                2023,
                2025,
                2026,
                2027,
                2029,
                2030,
                2031,
            }
            dt = (
                self._add_holiday_apr_14(sangkranta)
                if self._year in sangkranta_years_apr_14
                else self._add_holiday_apr_13(sangkranta)
            )
            self._add_holiday(sangkranta, _timedelta(dt, +1))
            self._add_holiday(sangkranta, _timedelta(dt, +2))

        #  ទិវាពលកម្មអន្តរជាតិ
        # Status: In-Use.

        # International Labor Day.
        self._add_labor_day(tr("ទិវាពលកម្មអន្តរជាតិ"))

        # ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី
        # Status: In-Use.
        # Assumed to start in 2005. Was celebrated for 3 days until 2020.

        if self._year >= 2005:
            king_sihamoni_bday = tr(
                # Birthday of His Majesty Preah Bat Samdech Preah Boromneath
                # NORODOM SIHAMONI, King of Cambodia.
                "ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម ព្រះករុណា ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី"
            )
            self._add_holiday_may_14(king_sihamoni_bday)
            if self._year <= 2019:
                self._add_holiday_may_13(king_sihamoni_bday)
                self._add_holiday_may_15(king_sihamoni_bday)

        # ទិវាជាតិនៃការចងចាំ
        # Status: Defunct.
        # Active between 2018-2019 as Public Holiday.
        # Was ទិវាចងកំហឹង (National Day of Anger) between 1983-2000.
        # Its celebration was put onhold by UN administration with
        # its name changed to present one in 2001.

        if 2018 <= self._year <= 2019:
            # National Day of Remembrance.
            self._add_holiday_may_20(tr("ទិវាជាតិនៃការចងចាំ"))

        # ទិវាកុមារអន្តរជាតិ
        # Status: Defunct.
        # Assumed to start in 1993, defunct from 2020 onwards.

        if self._year <= 2019:
            # International Children's Day.
            self._add_childrens_day(tr("ទិវាកុមារអន្តរជាតិ"))

        # ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ
        # Status: In-Use.
        # Assumed to start in 1994. A public holiday since 2015 at least.

        if self._year >= 1994:
            self._add_holiday_jun_18(
                # Birthday of Her Majesty the Queen-Mother NORODOM MONINEATH SIHANOUK of Cambodia.
                tr("ព្រះរាជពិធីបុណ្យចម្រើនព្រះជន្ម សម្តេចព្រះមហាក្សត្រី ព្រះវររាជមាតា នរោត្តម មុនិនាថ សីហនុ")
            )

        # ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ
        # Status: In-Use.
        # Starts in 1993

        # Constitution Day.
        self._add_holiday_sep_24(tr("ទិវាប្រកាសរដ្ឋធម្មនុញ្ញ"))

        # ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ នរោត្តម សីហនុ
        # ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី និងឯកភាពជាតិខ្មែរ ព្រះបរមរតនកោដ្ឋ
        # Status: In-Use.
        # Starts in 2012.

        if self._year >= 2012:
            self._add_holiday_oct_15(
                # Mourning Day of the Late King-Father NORODOM SIHANOUK of Cambodia.
                tr(
                    "ទិវាប្រារព្ឋពិធីគោរពព្រះវិញ្ញាណក្ខន្ឋ ព្រះករុណា ព្រះបាទសម្តេចព្រះ នរោត្តម "
                    "សីហនុ ព្រះមហាវីរក្សត្រ ព្រះវររាជបិតាឯករាជ្យ បូរណភាពទឹកដី និងឯកភាពជាតិខ្មែរ "
                    "ព្រះបរមរតនកោដ្ឋ"
                )
            )

        # ទិវារំលឹកសន្ធិសញ្ញាសន្តិភាពទីក្រុងប៉ារីស
        # Status: Defunct.
        # Assumed to start in 1993, defunct from 2020 onwards.

        if self._year <= 2019:
            # Paris Peace Agreement's Day.
            self._add_holiday_oct_23(tr("ទិវារំលឹកសន្ធិសញ្ញាសន្តិភាពទីក្រុងប៉ារីស"))

        # ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា
        # ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី
        # ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា
        # Status: In-Use.
        # Starts in 2004.

        if self._year >= 2004:
            self._add_holiday_oct_29(
                # Coronation Day of His Majesty Preah Bat Samdech Preah
                # Boromneath NORODOM SIHAMONI, King of Cambodia.
                tr(
                    "ព្រះរាជពិធីគ្រងព្រះបរមរាជសម្បត្តិ របស់ ព្រះករុណា "
                    "ព្រះបាទសម្តេចព្រះបរមនាថ នរោត្តម សីហមុនី "
                    "ព្រះមហាក្សត្រនៃព្រះរាជាណាចក្រកម្ពុជា"
                )
            )

        # ពិធីបុណ្យឯករាជ្យជាតិ
        # Status: In-Use.
        # Starts in 1953

        # National Independence Day.
        self._add_holiday_nov_9(tr("ពិធីបុណ្យឯករាជ្យជាតិ"))

        # ទិវាសិទ្ធិមនុស្សអន្តរជាតិ
        # Status: Defunct.
        # Assumed to start in 1993, defunct from 2020 onwards.

        if self._year <= 2019:
            # International Human Rights Day.
            self._add_holiday_dec_10(tr("ទិវាសិទ្ធិមនុស្សអន្តរជាតិ"))

        # Cambodian Lunar Calendar Holidays
        # See `_ThaiLunisolar` in holidays/calendars/thai.py for more details.
        # Cambodian Lunar Calendar Holidays only work from 1941 to 2157.

        # ពិធីបុណ្យមាឃបូជា
        # Status: Defunct.
        # 15th Waxing Day of Month 3.
        # Defunct from 2020 onwards.

        if self._year <= 2019:
            # Meak Bochea Day.
            self._add_makha_bucha(tr("ពិធីបុណ្យមាឃបូជា"))

        # ពិធីបុណ្យវិសាខបូជា
        # Status: In-Use.
        # 15th Waxing Day of Month 6.
        # This utilizes Thai calendar as a base, though are calculated to always happen
        # in the Traditional Visakhamas month (May).

        # Visaka Bochea Day.
        self._add_visakha_bucha(tr("ពិធីបុណ្យវិសាខបូជា"))

        # ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល
        # Status: In-Use.
        # 4th Waning Day of Month 6.
        # Unlike Thai ones, Cambodian Royal Ploughing Ceremony is always fixed.

        # Royal Ploughing Ceremony.
        self._add_preah_neangkoal(tr("ព្រះរាជពិធីច្រត់ព្រះនង្គ័ល"))

        # ពិធីបុណ្យភ្ផុំបិណ្ឌ
        # Status: In-Use.
        # 14th Waning Day of Month 10 - 1st Waxing Day of Month 11.
        # The 3rd day is added as a public holiday from 2017 onwards.

        # Pchum Ben Day.
        pchum_ben = tr("ពិធីបុណ្យភ្ផុំបិណ្ឌ")
        pchum_ben_date = self._add_pchum_ben(pchum_ben)
        self._add_holiday(pchum_ben, _timedelta(pchum_ben_date, -1))
        if self._year >= 2017:
            self._add_holiday(pchum_ben, _timedelta(pchum_ben_date, +1))

        # ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក
        # Status: In-Use.
        # 14th Waxing Day of Month 12 - 1st Waning Day of Month 12.

        # Water Festival.
        bon_om_touk = tr("ព្រះរាជពិធីបុណ្យអុំទូក បណ្តែតប្រទីប និងសំពះព្រះខែអកអំបុក")
        bon_om_touk_date = self._add_loy_krathong(bon_om_touk)
        self._add_holiday(bon_om_touk, _timedelta(bon_om_touk_date, -1))
        self._add_holiday(bon_om_touk, _timedelta(bon_om_touk_date, +1))

        # ទិវាសន្តិភាពនៅកម្ពុជា
        # Status: In-Use.
        # Dec 29, added from 2024 onwards.

        if self._year >= 2024:
            # Peace Day in Cambodia.
            self._add_holiday_dec_29(tr("ទិវាសន្តិភាពនៅកម្ពុជា"))


class KH(Cambodia):
    pass


class KHM(Cambodia):
    pass


class CambodiaStaticHolidays:
    sangkranta_in_lieu_covid = tr(
        # Khmer New Year's Replacement Holiday.
        "ថ្ងៃឈប់សម្រាកសងជំនួសឲ្យពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ"
    )
    # Special Public Holiday.
    special_in_lieu_holidays = tr("ថ្ងៃឈប់សម្រាកសងជំនួស")

    # Khmer New Year's Day.
    sangkranta = tr("ពិធីបុណ្យចូលឆ្នាំថ្មីប្រពៃណីជាតិ")

    special_public_holidays = {
        2016: (
            (MAY, 2, special_in_lieu_holidays),
            (MAY, 16, special_in_lieu_holidays),
        ),
        2018: (MAY, 21, special_in_lieu_holidays),
        2019: (SEP, 30, special_in_lieu_holidays),
        2020: (
            (MAY, 11, special_in_lieu_holidays),
            (AUG, 17, sangkranta_in_lieu_covid),
            (AUG, 18, sangkranta_in_lieu_covid),
            (AUG, 19, sangkranta_in_lieu_covid),
            (AUG, 20, sangkranta_in_lieu_covid),
            (AUG, 21, sangkranta_in_lieu_covid),
        ),
        2024: (APR, 16, sangkranta),
    }



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.gibraltar import Gibraltar
from tests.common import CommonCountryTests


class TestGibraltar(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Gibraltar)

    def test_special_holidays(self):
        for dt, name in (
            ("2004-08-04", "Tercentenary Holiday"),
            ("2012-06-05", "Queen's Diamond Jubilee"),
            ("2020-05-08", "75th Anniversary of VE Day"),
            ("2022-06-03", "Platinum Jubilee"),
            ("2009-01-12", "Bank Holiday"),
            ("2015-09-07", "Evacuation Commemoration Day"),
            ("2023-05-08", "Special King's Coronation Bank Holiday"),
        ):
            self.assertHolidayName(name, dt)

    def test_new_years_day(self):
        name = "New Year's Day"
        self.assertHolidayName(name, (f"{year}-01-01" for year in self.full_range))
        obs_dt = (
            "2012-01-02",
            "2017-01-02",
            "2022-01-03",
            "2023-01-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_winter_bank_holiday(self):
        name = "Winter Midterm Bank Holiday"
        self.assertHolidayName(
            name,
            "2023-02-20",
            "2024-02-12",
            "2025-02-17",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_commonwealth_day(self):
        name = "Commonwealth Day"
        self.assertHolidayName(
            name,
            "2018-03-12",
            "2019-03-11",
            "2020-03-09",
            "2021-02-15",
            "2022-02-21",
        )
        self.assertHolidayName(name, range(self.start_year, 2023))
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_good_friday(self):
        name = "Good Friday"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Easter Monday"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_workers_memorial_day(self):
        name = "Workers' Memorial Day"
        self.assertHolidayName(
            name,
            "2020-04-28",
            "2021-04-28",
            "2022-04-28",
            "2023-04-28",
            "2024-04-28",
            "2025-04-28",
        )
        self.assertHolidayName(name, range(2013, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2013))
        obs_dt = (
            "2018-04-30",
            "2019-04-29",
            "2024-04-29",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_may_day(self):
        name = "May Day"
        self.assertHolidayName(
            name,
            (
                f"{year}-05-01"
                for year in (*range(self.start_year, 2007), *range(2010, self.end_year))
            ),
        )
        self.assertHolidayName(
            name,
            "2007-05-07",
            "2008-05-05",
            "2009-05-04",
        )
        obs_dt = (
            "2011-05-02",
            "2016-05-02",
            "2021-05-03",
            "2022-05-02",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_spring_bank_holiday(self):
        name = "Spring Bank Holiday"
        self.assertHolidayName(
            name,
            "2020-05-25",
            "2021-05-31",
            "2022-06-02",
            "2023-05-29",
            "2024-05-27",
            "2025-05-26",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "2018-06-11",
            "2019-06-17",
            "2020-06-15",
            "2021-06-14",
            "2022-06-13",
        )
        self.assertHolidayName(name, range(self.start_year, 2023))
        self.assertNoHolidayName(name, range(2023, self.end_year))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "2023-06-19",
            "2024-06-17",
            "2025-06-16",
        )
        self.assertHolidayName(name, range(2023, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2023))

    def test_summer_bank_holiday(self):
        first_name = "Summer Bank Holiday"
        self.assertHolidayName(
            first_name,
            "2002-08-26",
            "2003-08-25",
            "2004-08-30",
            "2005-08-29",
            "2006-08-28",
            "2007-08-27",
        )
        self.assertNoHolidayName(
            first_name, range(self.start_year, 2002), range(2008, self.end_year)
        )

        second_name = "Late Summer Bank Holiday"
        self.assertHolidayName(
            second_name,
            "2020-08-31",
            "2021-08-30",
            "2022-08-29",
            "2023-08-28",
            "2024-08-26",
            "2025-08-25",
        )
        self.assertHolidayName(
            second_name, range(self.start_year, 2002), range(2008, self.end_year)
        )
        self.assertNoHolidayName(second_name, range(2002, 2008))

    def test_national_day(self):
        name = "Gibraltar National Day"
        self.assertHolidayName(
            name,
            (
                f"{year}-09-10"
                for year in (*range(self.start_year, 2016), *range(2018, self.end_year))
            ),
        )
        self.assertHolidayName(
            name,
            "2016-09-05",
            "2017-09-04",
        )
        obs_dt = (
            "2011-09-12",
            "2022-09-12",
            "2023-09-11",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_christmas_day(self):
        name = "Christmas Day"
        self.assertHolidayName(name, (f"{year}-12-25" for year in self.full_range))
        obs_dt = (
            "2011-12-27",
            "2016-12-27",
            "2021-12-27",
            "2022-12-27",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_boxing_day(self):
        name = "Boxing Day"
        self.assertHolidayName(name, (f"{year}-12-26" for year in self.full_range))
        obs_dt = (
            "2010-12-28",
            "2015-12-28",
            "2020-12-28",
            "2021-12-28",
        )
        self.assertHolidayName(f"{name} (observed)", obs_dt)
        self.assertNoNonObservedHoliday(obs_dt)

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "New Year's Day"),
            ("2024-02-12", "Winter Midterm Bank Holiday"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-28", "Workers' Memorial Day"),
            ("2024-04-29", "Workers' Memorial Day (observed)"),
            ("2024-05-01", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-06-17", "King's Birthday"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-09-10", "Gibraltar National Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-02-12", "Winter Midterm Bank Holiday"),
            ("2024-03-29", "Good Friday"),
            ("2024-04-01", "Easter Monday"),
            ("2024-04-28", "Workers' Memorial Day"),
            ("2024-04-29", "Workers' Memorial Day (observed)"),
            ("2024-05-01", "May Day"),
            ("2024-05-27", "Spring Bank Holiday"),
            ("2024-06-17", "King's Birthday"),
            ("2024-08-26", "Late Summer Bank Holiday"),
            ("2024-09-10", "Gibraltar National Day"),
            ("2024-12-25", "Christmas Day"),
            ("2024-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_MON,
    SAT_SUN_TO_NEXT_MON_TUE,
)


class Tuvalu(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Tuvalu holidays.

    References:
        * [Public holidays in Tuvalu](https://en.wikipedia.org/wiki/Public_holidays_in_Tuvalu)
        * [Today's and Upcoming Holidays in Tuvalu](https://web.archive.org/web/20250310164727/https://www.timeanddate.com/holidays/tuvalu/)
        * [Public Holidays (Amendment) Act 1990](https://web.archive.org/web/20250429073846/https://www.paclii.org/cgi-bin/sinodisp/tv/legis/num_act/pha1990243/pha1990243.html)
        * [Public Holidays Act 1](https://archive.org/details/tuvalu-public-holidays-act-revised-2008)
        * [Public Holidays (Amendment) Act 2018](https://archive.org/details/tuvalu-public-holidays-amendment-act-2018)
        * [Public Holidays (Amendment) Act 2020](https://archive.org/details/tuvalu-public-holidays-amendment-act-2020)
        * [Public Holidays Act 2](https://archive.org/details/tuvalu-public-holidays-act-revised-2022)
        * [Codes for the representation of names of countries and their subdivisions](https://www.iso.org/obp/ui/#iso:code:3166:TV)
        * [TUVALU-NEWS.TV](https://web.archive.org/web/20140915180104/http://www.tuvalu-news.tv/archives/2007/01/island_special_public_holidays.html)
    """

    country = "TV"
    default_language = "tvl"
    # %s (observed).
    observed_label = tr("%s (fakamatakuga)")
    # Tuvalu became fully independent of the United Kingdom on October 1, 1978
    # Tuvalu's PUBLIC HOLIDAYS (AMENDMENT) ACT 1990 (Act 2 of 1990)
    # It was first proclaimed on FEB 7th, 1990
    start_year = 1990
    subdivisions = (
        "FUN",  # Funafuti.
        "NIT",  # Niutao.
        "NKF",  # Nukufetau.
        "NKL",  # Nukulaelae.
        "NMA",  # Nanumea.
        "NMG",  # Nanumaga.
        "NUI",  # Nui.
        "VAI",  # Vaitupu.
    )
    subdivisions_aliases = {
        # Town/Island Councils.
        "Funafuti": "FUN",
        "Niutao": "NIT",
        "Nukufetau": "NKF",
        "Nukulaelae": "NKL",
        "Nanumea": "NMA",
        "Nanumaga": "NMG",
        "Nanumanga": "NMG",  # Ex-ISO code alias
        "Nui": "NUI",
        "Vaitupu": "VAI",
    }
    supported_languages = ("en_GB", "en_US", "tvl")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_MON)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("Tausaga Fou")))

        if self._year <= 2020:
            # Commonwealth Day.
            self._add_holiday_2nd_mon_of_mar(tr("Aso Atefenua"))

        # Good Friday.
        self._add_good_friday(tr("Aso toe tu"))

        # Easter Monday.
        self._add_easter_monday(tr("Toe Tu aso gafua"))

        # Gospel Day.
        self._add_holiday_1_day_past_2nd_sun_of_may(tr("Te Aso o te Tala Lei"))

        self._add_holiday_2nd_sat_of_jun(
            # King's Birthday.
            tr("Asofanau Tupu")
            if self._year >= 2023
            # Queen's Birthday.
            else tr("Asofanau Fafine")
        )

        # National Children's Day.
        name = tr("Aso Tamaliki")
        if self._year >= 2019:
            # National Youth Day.
            self._add_holiday_1st_mon_of_aug(tr("Aso tupulaga"))

            # National Children's Day.
            self._add_holiday_1_day_past_2nd_sun_of_oct(name)
        else:
            # National Children's Day.
            self._add_holiday_1st_mon_of_aug(name)

        # Tuvalu Day.
        name = tr("Tutokotasi")
        self._add_observed(self._add_holiday_oct_1(name), rule=SAT_SUN_TO_NEXT_MON_TUE)
        self._add_observed(self._add_holiday_oct_2(name), rule=SAT_SUN_TO_NEXT_MON_TUE)

        # https://en.wikipedia.org/wiki/King's_Official_Birthday#Tuvalu
        if self._year <= 2022:
            # Heir to the Throne's Birthday.
            self._add_holiday_2nd_mon_of_nov(tr("Aso fanau o te sui ote Tupu"))

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Kilisimasi")), rule=SAT_SUN_TO_NEXT_MON_TUE)
        self._add_observed(
            # Boxing Day.
            self._add_christmas_day_two(tr("Aso Faipele")),
            rule=SAT_SUN_TO_NEXT_MON_TUE,
        )

    def _populate_subdiv_fun_public_holidays(self):
        # The Day of the Bombing.
        self._add_holiday_apr_23(tr("Te Aso o te Paula"))

        # Cyclone Day.
        self._add_holiday_oct_21(tr("Aso o te matagi"))

    def _populate_subdiv_nit_public_holidays(self):
        # Niutao Day.
        self._add_holiday_sep_17(tr("Te Aso o te Setema"))

    def _populate_subdiv_nkf_public_holidays(self):
        # Nukufetau Day.
        self._add_holiday_feb_11(tr("Te Aso O Tutasi"))

    def _populate_subdiv_nkl_public_holidays(self):
        # Gospel Day.
        self._add_holiday_may_10(tr("Te Aso o te Tala Lei"))

    def _populate_subdiv_nma_public_holidays(self):
        # Golden Jubilee.
        self._add_holiday_jan_8(tr("Te Po o Tefolaha"))

        # Big Day.
        self._add_holiday_feb_3(tr("Po Lahi"))

    def _populate_subdiv_nmg_public_holidays(self):
        # Nanumaga Day.
        self._add_holiday_apr_15(tr("Aho o te Fakavae"))

    def _populate_subdiv_nui_public_holidays(self):
        # Day of the Flood.
        self._add_holiday_feb_16(tr("Bogin te Ieka"))

    def _populate_subdiv_vai_public_holidays(self):
        # Happy Day.
        self._add_holiday_nov_25(tr("Te Aso Fiafia"))


class TV(Tuvalu):
    pass


class TUV(Tuvalu):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import OPTIONAL
from holidays.countries.netherlands import Netherlands
from tests.common import CommonCountryTests


class TestNetherlands(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Netherlands)

    def test_no_holidays(self):
        super().test_no_holidays()

        self.assertNoHolidays(Netherlands(categories=OPTIONAL, years=range(self.start_year, 1982)))

    def test_new_years_day(self):
        self.assertHolidayName("Nieuwjaarsdag", (f"{year}-01-01" for year in self.full_range))

    def test_good_friday(self):
        name = "Goede Vrijdag"
        self.assertHolidayName(
            name,
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            "2024-03-29",
            "2025-04-18",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_sunday(self):
        name = "Eerste paasdag"
        self.assertHolidayName(
            name,
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            "2024-03-31",
            "2025-04-20",
        )
        self.assertHolidayName(name, self.full_range)

    def test_easter_monday(self):
        name = "Tweede paasdag"
        self.assertHolidayName(
            name,
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            "2024-04-01",
            "2025-04-21",
        )
        self.assertHolidayName(name, self.full_range)

    def test_queens_day(self):
        name = "Koninginnedag"
        self.assertHolidayName(
            name,
            "1967-05-01",
            "1972-05-01",
            "1978-05-01",
            "1989-04-29",
            "1995-04-29",
            "2000-04-29",
            "2006-04-29",
            "2013-04-30",
        )
        self.assertHolidayName(name, range(self.start_year, 2014))
        self.assertNoHolidayName(
            name,
            "1967-04-30",
            "1972-04-30",
            "1978-04-30",
            "1995-04-30",
            "1989-04-30",
            "2000-04-30",
            "2006-04-30",
        )
        self.assertNoHolidayName(name, range(2014, self.end_year))

    def test_king_day(self):
        name = "Koningsdag"
        self.assertHolidayName(
            name,
            "2016-04-27",
            "2017-04-27",
            "2018-04-27",
            "2019-04-27",
            "2020-04-27",
            "2021-04-27",
            "2022-04-27",
            "2023-04-27",
            "2024-04-27",
            "2025-04-26",
            "2031-04-26",
            "2036-04-26",
        )
        self.assertHolidayName(name, range(2014, self.end_year))
        self.assertNoHolidayName(
            name,
            "2014-04-27",
            "2025-04-27",
            "2031-04-27",
            "2036-04-27",
        )
        self.assertNoHolidayName(name, range(self.start_year, 2014))

    def test_liberation_day(self):
        name = "Bevrijdingsdag"
        # PUBLIC.
        self.assertHolidayName(
            name, (f"{year}-05-05" for year in self.full_range if year % 5 == 0)
        )
        self.assertNoHolidayName(
            name, (f"{year}-05-05" for year in self.full_range if year % 5 != 0)
        )
        # OPTIONAL.
        self.assertOptionalHolidayName(
            name, (f"{year}-05-05" for year in range(1982, self.end_year))
        )
        self.assertNoOptionalHolidayName(name, range(self.start_year, 1982))

    def test_ascension_day(self):
        name = "Hemelvaartsdag"
        self.assertHolidayName(
            name,
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            "2024-05-09",
            "2025-05-29",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_sunday(self):
        name = "Eerste Pinksterdag"
        self.assertHolidayName(
            name,
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            "2024-05-19",
            "2025-06-08",
        )
        self.assertHolidayName(name, self.full_range)

    def test_whit_monday(self):
        name = "Tweede Pinksterdag"
        self.assertHolidayName(
            name,
            "2020-06-01",
            "2021-05-24",
            "2022-06-06",
            "2023-05-29",
            "2024-05-20",
            "2025-06-09",
        )
        self.assertHolidayName(name, self.full_range)

    def test_christmas_day(self):
        self.assertHolidayName("Eerste Kerstdag", (f"{year}-12-25" for year in self.full_range))

    def test_second_day_of_christmas(self):
        self.assertHolidayName("Tweede Kerstdag", (f"{year}-12-26" for year in self.full_range))

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Nieuwjaarsdag"),
            ("2022-04-15", "Goede Vrijdag"),
            ("2022-04-17", "Eerste paasdag"),
            ("2022-04-18", "Tweede paasdag"),
            ("2022-04-27", "Koningsdag"),
            ("2022-05-05", "Bevrijdingsdag"),
            ("2022-05-26", "Hemelvaartsdag"),
            ("2022-06-05", "Eerste Pinksterdag"),
            ("2022-06-06", "Tweede Pinksterdag"),
            ("2022-12-25", "Eerste Kerstdag"),
            ("2022-12-26", "Tweede Kerstdag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "King's Day"),
            ("2022-05-05", "Liberation Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Whit Monday"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_fy(self):
        self.assertLocalizedHolidays(
            "fy",
            ("2022-01-01", "Nijjiersdei"),
            ("2022-04-15", "Goedfreed"),
            ("2022-04-17", "Peaskesnein"),
            ("2022-04-18", "Peaskemoandei"),
            ("2022-04-27", "Keningsdei"),
            ("2022-05-05", "Befrijingsdei"),
            ("2022-05-26", "Himelfeartsdei"),
            ("2022-06-05", "Pinkstersnein"),
            ("2022-06-06", "Pinkstermoandei"),
            ("2022-12-25", "Eerste Krystdei"),
            ("2022-12-26", "Twadde Krystdei"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2022-01-01", "วันขึ้นปีใหม่"),
            ("2022-04-15", "วันศุกร์ประเสริฐ"),
            ("2022-04-17", "วันอาทิตย์อีสเตอร์"),
            ("2022-04-18", "วันจันทร์อีสเตอร์"),
            ("2022-04-27", "วันเฉลิมพระชนมพรรษาสมเด็จพระราชาธิบดี"),
            ("2022-05-05", "วันประกาศอิสรภาพ"),
            ("2022-05-26", "วันสมโภชพระเยซูเจ้าเสด็จขึ้นสวรรค์"),
            ("2022-06-05", "วันสมโภชพระจิตเจ้า"),
            ("2022-06-06", "วันจันทร์หลังวันสมโภชพระจิตเจ้า"),
            ("2022-12-25", "วันคริสต์มาสวันแรก"),
            ("2022-12-26", "วันคริสต์มาสวันที่สอง"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "День короля"),
            ("2022-05-05", "День визволення"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.mongolia import Mongolia
from tests.common import CommonCountryTests


class TestMongolia(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Mongolia)

    def test_new_years_day(self):
        self.assertHolidayName("Шинэ жил", (f"{year}-01-01" for year in self.full_range))

    def test_lunar_new_year(self):
        name = "Цагаан сар"
        # Day 1.
        self.assertHolidayName(
            name,
            "2021-02-12",
            "2022-02-02",
            "2023-02-21",
            "2024-02-10",
            "2025-03-01",
        )
        # Day 2.
        self.assertHolidayName(
            name,
            "2021-02-13",
            "2022-02-03",
            "2023-02-22",
            "2024-02-11",
            "2025-03-02",
        )
        # Day 3.
        self.assertHolidayName(
            name,
            "2021-02-14",
            "2022-02-04",
            "2023-02-23",
            "2024-02-12",
            "2025-03-03",
        )
        self.assertHolidayNameCount(name, 2, range(self.start_year, 2014))
        self.assertHolidayNameCount(name, 3, range(2014, self.end_year))

    def test_international_womens_day(self):
        self.assertHolidayName(
            "Олон улсын эмэгтэйчүүдийн өдөр", (f"{year}-03-08" for year in self.full_range)
        )

    def test_childrens_day(self):
        self.assertHolidayName("Хүүхдийн баяр", (f"{year}-06-01" for year in self.full_range))

    def test_the_buddhas_birthday(self):
        name = "Бурхан багшийн Их дүйчин өдөр"
        self.assertHolidayName(
            name,
            "2021-05-26",
            "2022-06-14",
            "2023-06-04",
            "2024-05-23",
            "2025-06-11",
        )
        self.assertHolidayName(name, range(2020, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2020))

    def test_national_festival_and_peoples_revolution_anniversary(self):
        name = "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"

        # JUL 10.
        self.assertHolidayName(name, (f"{year}-07-10" for year in range(2023, self.end_year)))
        self.assertNoHolidayName(name, (f"{year}-07-10" for year in range(self.start_year, 2023)))

        # JUL 11, 12, 13.
        for year in self.full_range:
            self.assertHolidayName(name, f"{year}-07-11", f"{year}-07-12", f"{year}-07-13")

        # JUL 14, 15.
        for year in range(2014, self.end_year):
            self.assertHolidayName(name, f"{year}-07-14", f"{year}-07-15")
        for year in range(self.start_year, 2014):
            self.assertNoHolidayName(name, f"{year}-07-14", f"{year}-07-15")

    def test_genghis_khans_birthday(self):
        name = "Их Эзэн Чингис хааны өдөр"
        self.assertHolidayName(
            name,
            "2021-11-05",
            "2022-11-24",
            "2023-11-14",
            "2024-11-02",
            "2025-11-21",
        )
        self.assertHolidayName(name, range(2012, self.end_year))
        self.assertNoHolidayName(name, range(self.start_year, 2012))

    def test_republic_day(self):
        name_2004 = "Бүгд Найрамдах Улс тунхагласны баяр"
        name_2012 = "Бүгд Найрамдах Улс тунхагласан өдөр"

        # Public holidays.
        self.assertHolidayName(
            name_2004, (f"{year}-11-26" for year in range(self.start_year, 2012))
        )
        self.assertHolidayName(name_2012, (f"{year}-11-26" for year in range(2016, self.end_year)))
        self.assertNoHolidayName(name_2004, range(2012, self.end_year))
        self.assertNoHolidayName(name_2012, range(self.start_year, 2016))

        # Workday holidays.
        self.assertWorkdayHolidayName(name_2012, (f"{year}-11-26" for year in range(2012, 2016)))
        self.assertNoWorkdayHolidayName(
            name_2012, range(self.start_year, 2012), range(2016, self.end_year)
        )

    def test_independence_day(self):
        name_2007 = "Үндэсний эрх чөлөөний өдөр"
        name_2011 = "Үндэсний эрх чөлөө, тусгаар тогтнолоо сэргээсний баярын өдөр"

        # Public holidays.
        self.assertHolidayName(name_2011, (f"{year}-12-29" for year in range(2011, self.end_year)))
        self.assertNoHolidayName(name_2007)
        self.assertNoHolidayName(name_2011, range(self.start_year, 2011))

        # Workday holidays.
        self.assertWorkdayHolidayName(name_2007, (f"{year}-12-29" for year in range(2007, 2011)))
        self.assertNoWorkdayHolidayName(
            name_2007, range(self.start_year, 2007), range(2011, self.end_year)
        )
        self.assertNoWorkdayHolidayName(name_2011)

    def test_constitutional_day(self):
        name = "Монгол Улсын Үндсэн хуулийн өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-01-13" for year in self.full_range))

    def test_patriots_day(self):
        name = "Эх орончдын өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-03-01" for year in range(2005, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2005))

    def test_military_day(self):
        name_2004 = "Зэвсэгт хүчний өдөр"
        name_2011 = "Монгол цэргийн өдөр"
        self.assertNoHolidayName(name_2004)
        self.assertNoHolidayName(name_2011)
        self.assertWorkdayHolidayName(
            name_2004, (f"{year}-03-18" for year in range(self.start_year, 2011))
        )
        self.assertWorkdayHolidayName(
            name_2011, (f"{year}-03-18" for year in range(2011, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name_2004, range(2011, self.end_year))
        self.assertNoWorkdayHolidayName(name_2011, range(self.start_year, 2011))

    def test_health_protection_day(self):
        name = "Эрүүл мэндийг хамгаалах өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-04-07" for year in self.full_range))

    def test_intellectual_property_protection_day(self):
        name = "Оюуны өмчийг хамгаалах өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-04-26" for year in self.full_range))

    def test_family_day(self):
        name = "Гэр бүлийн өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-05-15" for year in self.full_range))

    def test_national_flag_day(self):
        name = "Монгол Улсын төрийн далбааны өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name, (f"{year}-07-10" for year in range(2009, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2009))

    def test_youth_day(self):
        name = "Залуучуудын өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-08-25" for year in self.full_range))

    def test_new_harvest_days(self):
        name = "Шинэ ургацын өдрүүд"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            (f"{year}-09-05" for year in range(2007, self.end_year)),
            (f"{year}-10-01" for year in range(2007, self.end_year)),
            (f"{year}-10-20" for year in range(2007, self.end_year)),
        )
        self.assertWorkdayHolidayNameCount(name, 46, range(2007, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2007))

    def test_memorial_day_of_political_victims(self):
        name = "Улс төрийн хэлмэгдэгсдийн дурсгалын өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-09-10" for year in self.full_range))

    def test_national_literary_cultural_and_book_days(self):
        name = "Үндэсний бичиг соёл, номын өдрүүд"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2021-09-18",
            "2021-09-19",
            "2022-05-14",
            "2022-05-15",
            "2022-09-17",
            "2022-09-18",
            "2023-05-20",
            "2023-05-21",
            "2023-09-16",
            "2023-09-17",
            "2024-05-18",
            "2024-05-19",
            "2024-09-14",
            "2024-09-15",
            "2025-05-17",
            "2025-05-18",
            "2025-09-20",
            "2025-09-21",
            "2026-05-16",
        )
        self.assertWorkdayHolidayNameCount(name, 2, 2021)
        self.assertWorkdayHolidayNameCount(name, 4, range(2022, self.end_year))
        self.assertNoWorkdayHolidayName(name, range(self.start_year, 2021))

    def test_environmental_protection_days(self):
        name = "Байгаль орчныг хамгаалах өдрүүд"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(
            name,
            "2023-09-18",
            "2023-09-19",
            "2023-09-20",
            "2023-09-21",
            "2023-09-22",
            "2023-09-23",
            "2023-09-24",
            "2024-09-16",
            "2024-09-17",
            "2024-09-18",
            "2024-09-19",
            "2024-09-20",
            "2024-09-21",
            "2024-09-22",
            "2025-09-22",
            "2025-09-23",
            "2025-09-24",
            "2025-09-25",
            "2025-09-26",
            "2025-09-27",
            "2025-09-28",
        )
        self.assertWorkdayHolidayNameCount(name, 7, self.full_range)

    def test_elders_day(self):
        name = "Ахмадын өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-10-01" for year in self.full_range))

    def test_capital_day(self):
        name_2004 = "Монгол Улсын Нийслэлийн өдөр"
        name_2021 = "Монгол Улсын нийслэл хотын өдөр"
        self.assertNoHolidayName(name_2004)
        self.assertNoHolidayName(name_2021)
        self.assertWorkdayHolidayName(
            name_2004, (f"{year}-10-29" for year in range(self.start_year, 2021))
        )
        self.assertWorkdayHolidayName(
            name_2021, (f"{year}-10-29" for year in range(2021, self.end_year))
        )
        self.assertNoWorkdayHolidayName(name_2004, range(2021, self.end_year))
        self.assertNoWorkdayHolidayName(name_2021, range(self.start_year, 2021))

    def test_democracy_and_human_rights_day(self):
        name = "Ардчилал, хүний эрхийн өдөр"
        self.assertNoHolidayName(name)
        self.assertWorkdayHolidayName(name, (f"{year}-12-10" for year in self.full_range))

    def test_2025(self):
        self.assertHolidaysInYear(
            2025,
            ("2025-01-01", "Шинэ жил"),
            ("2025-03-01", "Цагаан сар"),
            ("2025-03-02", "Цагаан сар"),
            ("2025-03-03", "Цагаан сар"),
            ("2025-03-08", "Олон улсын эмэгтэйчүүдийн өдөр"),
            ("2025-06-01", "Хүүхдийн баяр"),
            ("2025-06-11", "Бурхан багшийн Их дүйчин өдөр"),
            ("2025-07-10", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-07-11", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-07-12", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-07-13", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-07-14", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-07-15", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2025-11-21", "Их Эзэн Чингис хааны өдөр"),
            ("2025-11-26", "Бүгд Найрамдах Улс тунхагласан өдөр"),
            ("2025-12-29", "Үндэсний эрх чөлөө, тусгаар тогтнолоо сэргээсний баярын өдөр"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2024-01-01", "Шинэ жил"),
            ("2024-01-13", "Монгол Улсын Үндсэн хуулийн өдөр"),
            ("2024-02-10", "Цагаан сар"),
            ("2024-02-11", "Цагаан сар"),
            ("2024-02-12", "Цагаан сар"),
            ("2024-03-01", "Эх орончдын өдөр"),
            ("2024-03-08", "Олон улсын эмэгтэйчүүдийн өдөр"),
            ("2024-03-18", "Монгол цэргийн өдөр"),
            ("2024-04-07", "Эрүүл мэндийг хамгаалах өдөр"),
            ("2024-04-26", "Оюуны өмчийг хамгаалах өдөр"),
            ("2024-05-15", "Гэр бүлийн өдөр"),
            ("2024-05-18", "Үндэсний бичиг соёл, номын өдрүүд"),
            ("2024-05-19", "Үндэсний бичиг соёл, номын өдрүүд"),
            ("2024-05-23", "Бурхан багшийн Их дүйчин өдөр"),
            ("2024-06-01", "Хүүхдийн баяр"),
            (
                "2024-07-10",
                "Монгол Улсын төрийн далбааны өдөр; "
                "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр",
            ),
            ("2024-07-11", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2024-07-12", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2024-07-13", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2024-07-14", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2024-07-15", "Үндэсний их баяр наадам, Ардын хувьсгалын ойн баяр"),
            ("2024-08-25", "Залуучуудын өдөр"),
            ("2024-09-05", "Шинэ ургацын өдрүүд"),
            ("2024-09-06", "Шинэ ургацын өдрүүд"),
            ("2024-09-07", "Шинэ ургацын өдрүүд"),
            ("2024-09-08", "Шинэ ургацын өдрүүд"),
            ("2024-09-09", "Шинэ ургацын өдрүүд"),
            ("2024-09-10", "Улс төрийн хэлмэгдэгсдийн дурсгалын өдөр; Шинэ ургацын өдрүүд"),
            ("2024-09-11", "Шинэ ургацын өдрүүд"),
            ("2024-09-12", "Шинэ ургацын өдрүүд"),
            ("2024-09-13", "Шинэ ургацын өдрүүд"),
            ("2024-09-14", "Шинэ ургацын өдрүүд; Үндэсний бичиг соёл, номын өдрүүд"),
            ("2024-09-15", "Шинэ ургацын өдрүүд; Үндэсний бичиг соёл, номын өдрүүд"),
            ("2024-09-16", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-17", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-18", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-19", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-20", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-21", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-22", "Байгаль орчныг хамгаалах өдрүүд; Шинэ ургацын өдрүүд"),
            ("2024-09-23", "Шинэ ургацын өдрүүд"),
            ("2024-09-24", "Шинэ ургацын өдрүүд"),
            ("2024-09-25", "Шинэ ургацын өдрүүд"),
            ("2024-09-26", "Шинэ ургацын өдрүүд"),
            ("2024-09-27", "Шинэ ургацын өдрүүд"),
            ("2024-09-28", "Шинэ ургацын өдрүүд"),
            ("2024-09-29", "Шинэ ургацын өдрүүд"),
            ("2024-09-30", "Шинэ ургацын өдрүүд"),
            ("2024-10-01", "Ахмадын өдөр; Шинэ ургацын өдрүүд"),
            ("2024-10-02", "Шинэ ургацын өдрүүд"),
            ("2024-10-03", "Шинэ ургацын өдрүүд"),
            ("2024-10-04", "Шинэ ургацын өдрүүд"),
            ("2024-10-05", "Шинэ ургацын өдрүүд"),
            ("2024-10-06", "Шинэ ургацын өдрүүд"),
            ("2024-10-07", "Шинэ ургацын өдрүүд"),
            ("2024-10-08", "Шинэ ургацын өдрүүд"),
            ("2024-10-09", "Шинэ ургацын өдрүүд"),
            ("2024-10-10", "Шинэ ургацын өдрүүд"),
            ("2024-10-11", "Шинэ ургацын өдрүүд"),
            ("2024-10-12", "Шинэ ургацын өдрүүд"),
            ("2024-10-13", "Шинэ ургацын өдрүүд"),
            ("2024-10-14", "Шинэ ургацын өдрүүд"),
            ("2024-10-15", "Шинэ ургацын өдрүүд"),
            ("2024-10-16", "Шинэ ургацын өдрүүд"),
            ("2024-10-17", "Шинэ ургацын өдрүүд"),
            ("2024-10-18", "Шинэ ургацын өдрүүд"),
            ("2024-10-19", "Шинэ ургацын өдрүүд"),
            ("2024-10-20", "Шинэ ургацын өдрүүд"),
            ("2024-10-29", "Монгол Улсын нийслэл хотын өдөр"),
            ("2024-11-02", "Их Эзэн Чингис хааны өдөр"),
            ("2024-11-26", "Бүгд Найрамдах Улс тунхагласан өдөр"),
            ("2024-12-10", "Ардчилал, хүний эрхийн өдөр"),
            ("2024-12-29", "Үндэсний эрх чөлөө, тусгаар тогтнолоо сэргээсний баярын өдөр"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2024-01-01", "New Year's Day"),
            ("2024-01-13", "Constitution Day"),
            ("2024-02-10", "Lunar New Year"),
            ("2024-02-11", "Lunar New Year"),
            ("2024-02-12", "Lunar New Year"),
            ("2024-03-01", "Patriots' Day"),
            ("2024-03-08", "International Women's Day"),
            ("2024-03-18", "Military Day"),
            ("2024-04-07", "Health Protection Day"),
            ("2024-04-26", "Intellectual Property Protection Day"),
            ("2024-05-15", "Family Day"),
            ("2024-05-18", "National Literary, Cultural and Book Days"),
            ("2024-05-19", "National Literary, Cultural and Book Days"),
            ("2024-05-23", "The Buddha's Birthday"),
            ("2024-06-01", "Children's Day"),
            (
                "2024-07-10",
                "National Festival and People's Revolution Anniversary; National Flag Day",
            ),
            ("2024-07-11", "National Festival and People's Revolution Anniversary"),
            ("2024-07-12", "National Festival and People's Revolution Anniversary"),
            ("2024-07-13", "National Festival and People's Revolution Anniversary"),
            ("2024-07-14", "National Festival and People's Revolution Anniversary"),
            ("2024-07-15", "National Festival and People's Revolution Anniversary"),
            ("2024-08-25", "Youth Day"),
            ("2024-09-05", "New Harvest Days"),
            ("2024-09-06", "New Harvest Days"),
            ("2024-09-07", "New Harvest Days"),
            ("2024-09-08", "New Harvest Days"),
            ("2024-09-09", "New Harvest Days"),
            ("2024-09-10", "Memorial Day of Political Victims; New Harvest Days"),
            ("2024-09-11", "New Harvest Days"),
            ("2024-09-12", "New Harvest Days"),
            ("2024-09-13", "New Harvest Days"),
            ("2024-09-14", "National Literary, Cultural and Book Days; New Harvest Days"),
            ("2024-09-15", "National Literary, Cultural and Book Days; New Harvest Days"),
            ("2024-09-16", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-17", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-18", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-19", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-20", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-21", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-22", "Environmental Protection Days; New Harvest Days"),
            ("2024-09-23", "New Harvest Days"),
            ("2024-09-24", "New Harvest Days"),
            ("2024-09-25", "New Harvest Days"),
            ("2024-09-26", "New Harvest Days"),
            ("2024-09-27", "New Harvest Days"),
            ("2024-09-28", "New Harvest Days"),
            ("2024-09-29", "New Harvest Days"),
            ("2024-09-30", "New Harvest Days"),
            ("2024-10-01", "Elders' Day; New Harvest Days"),
            ("2024-10-02", "New Harvest Days"),
            ("2024-10-03", "New Harvest Days"),
            ("2024-10-04", "New Harvest Days"),
            ("2024-10-05", "New Harvest Days"),
            ("2024-10-06", "New Harvest Days"),
            ("2024-10-07", "New Harvest Days"),
            ("2024-10-08", "New Harvest Days"),
            ("2024-10-09", "New Harvest Days"),
            ("2024-10-10", "New Harvest Days"),
            ("2024-10-11", "New Harvest Days"),
            ("2024-10-12", "New Harvest Days"),
            ("2024-10-13", "New Harvest Days"),
            ("2024-10-14", "New Harvest Days"),
            ("2024-10-15", "New Harvest Days"),
            ("2024-10-16", "New Harvest Days"),
            ("2024-10-17", "New Harvest Days"),
            ("2024-10-18", "New Harvest Days"),
            ("2024-10-19", "New Harvest Days"),
            ("2024-10-20", "New Harvest Days"),
            ("2024-10-29", "Capital City Day"),
            ("2024-11-02", "Genghis Khan's Birthday"),
            ("2024-11-26", "Republic Day"),
            ("2024-12-10", "Democracy and Human Rights Day"),
            ("2024-12-29", "National Freedom and Independence Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from gettext import gettext as tr

from holidays.groups import ChristianHolidays, InternationalHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    TUE_TO_PREV_MON,
    THU_TO_NEXT_FRI,
    SUN_TO_NEXT_MON,
)


class Bolivia(ObservedHolidayBase, ChristianHolidays, InternationalHolidays):
    """Bolivia holidays.

    References:
        * [Supreme Decree #14260](https://web.archive.org/web/20250428205639/https://bolivia.infoleyes.com/norma/1141/decreto-supremo-14260)
        * [Supreme Decree #21060](https://web.archive.org/web/20250428210031/https://bolivia.infoleyes.com/norma/1211/decreto-supremo-21060)
        * [Supreme Decree #22352](https://web.archive.org/web/20250428210103/https://bolivia.infoleyes.com/norma/1310/decreto-supremo-22352)
        * [Supreme Decree #0173](https://web.archive.org/web/20250428210129/https://bolivia.infoleyes.com/norma/829/decreto-supremo-0173)
        * [Supreme Decree #0405](https://web.archive.org/web/20250428210132/https://bolivia.infoleyes.com/norma/1252/decreto-supremo-0405)
        * [Supreme Decree #1210](https://web.archive.org/web/20250428210105/https://bolivia.infoleyes.com/norma/3756/decreto-supremo-1210)
        * [Supreme Decree #2750](https://web.archive.org/web/20250428210116/https://bolivia.infoleyes.com/norma/6023/decreto-supremo-2750)
        * <https://en.wikipedia.org/wiki/Public_holidays_in_Bolivia>
        * <https://web.archive.org/web/20250120171827/https://www.officeholidays.com/countries/bolivia>
    """

    country = "BO"
    default_language = "es"
    # %s (observed).
    observed_label = tr("%s (observado)")
    start_year = 1825
    subdivisions = (
        "B",  # El Beni.
        "C",  # Cochabamba.
        "H",  # Chuquisaca.
        "L",  # La Paz.
        "N",  # Pando.
        "O",  # Oruro.
        "P",  # Potosí.
        "S",  # Santa Cruz.
        "T",  # Tarija.
    )
    subdivisions_aliases = {
        "El Beni": "B",
        "Cochabamba": "C",
        "Chuquisaca": "H",
        "La Paz": "L",
        "Pando": "N",
        "Oruro": "O",
        "Potosí": "P",
        "Santa Cruz": "S",
        "Tarija": "T",
    }
    supported_languages = ("en_US", "es", "uk")

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        # Supreme Decree #14260.
        kwargs.setdefault("observed_rule", SUN_TO_NEXT_MON)
        kwargs.setdefault("observed_since", 1977)
        super().__init__(*args, **kwargs)

    def _populate_public_holidays(self):
        # New Year's Day.
        self._add_observed(self._add_new_years_day(tr("Año Nuevo")))

        # Supreme Decree #0405.
        if self._year >= 2010:
            self._add_observed(
                self._add_holiday_jan_22(
                    # Plurinational State Foundation Day.
                    tr("Día de la Creación del Estado Plurinacional de Bolivia")
                )
            )

        # Carnival.
        name = tr("Carnaval")
        self._add_carnival_monday(name)
        self._add_carnival_tuesday(name)

        # Good Friday.
        self._add_good_friday(tr("Viernes Santo"))

        # Labor Day.
        self._add_observed(may_1 := self._add_labor_day(tr("Día del Trabajo")))
        # Supreme Decree #1210.
        if 2012 <= self._year <= 2015:
            self._add_observed(may_1, rule=TUE_TO_PREV_MON + THU_TO_NEXT_FRI)

        # Corpus Christi.
        self._add_corpus_christi_day(tr("Corpus Christi"))

        # Supreme Decree #0173.
        if self._year >= 2009:
            # Aymara New Year.
            self._add_observed(self._add_holiday_jun_21(tr("Año Nuevo Aymara Amazónico")))

        # Independence Day.
        self._add_observed(self._add_holiday_aug_6(tr("Día de la Independencia de Bolivia")))

        if self._year >= 2020:
            # National Dignity Day.
            self._add_holiday_oct_17(tr("Día de la Dignidad Nacional"))

        # Supreme Decree #21060.
        if 1985 <= self._year <= 1988:
            # All Saints' Day.
            self._add_all_saints_day(tr("Día de Todos los Santos"))

        # Supreme Decree #22352.
        if self._year >= 1989:
            # All Souls' Day.
            nov_2 = self._add_all_souls_day(tr("Día de Todos los Difuntos"))
            if self._year <= 2015:
                self._add_observed(nov_2)

        # Christmas Day.
        self._add_observed(self._add_christmas_day(tr("Navidad")))

    def _populate_subdiv_b_public_holidays(self):
        # Beni Day.
        self._add_holiday_nov_18(tr("Día del departamento de Beni"))

    def _populate_subdiv_c_public_holidays(self):
        # Cochabamba Day.
        self._add_holiday_sep_14(tr("Día del departamento de Cochabamba"))

    def _populate_subdiv_h_public_holidays(self):
        # Chuquisaca Day.
        self._add_holiday_may_25(tr("Día del departamento de Chuquisaca"))

    def _populate_subdiv_l_public_holidays(self):
        # La Paz Day.
        self._add_holiday_jul_16(tr("Día del departamento de La Paz"))

    def _populate_subdiv_n_public_holidays(self):
        # Pando Day.
        self._add_holiday_oct_11(tr("Día del departamento de Pando"))

    def _populate_subdiv_p_public_holidays(self):
        # Potosí Day.
        self._add_holiday_nov_10(tr("Día del departamento de Potosí"))

    def _populate_subdiv_o_public_holidays(self):
        # Carnival in Oruro.
        self._add_holiday_51_days_prior_easter(tr("Carnaval de Oruro"))

    def _populate_subdiv_s_public_holidays(self):
        # Santa Cruz Day.
        self._add_holiday_sep_24(tr("Día del departamento de Santa Cruz"))

    def _populate_subdiv_t_public_holidays(self):
        # La Tablada.
        self._add_holiday_apr_15(tr("La Tablada"))


class BO(Bolivia):
    pass


class BOL(Bolivia):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see CONTRIBUTORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.poland import Poland
from tests.common import CommonCountryTests


class TestPoland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Poland, years=range(1925, 2050))

    def test_special_holidays(self):
        self.assertHoliday("2018-11-12")

    def test_new_years_day(self):
        self.assertHolidayName("Nowy Rok", (f"{year}-01-01" for year in range(1925, 2050)))

    def test_epiphany(self):
        name = "Święto Trzech Króli"
        self.assertHolidayName(
            name, (f"{year}-01-06" for year in (*range(1925, 1960), *range(2011, 2050)))
        )
        self.assertNoHoliday(f"{year}-01-06" for year in range(1961, 2011))
        self.assertNoHolidayName(name, range(1961, 2011))

    def test_candlemas(self):
        name = "Oczyszczenie Najświętszej Marii Panny"
        self.assertHolidayName(name, (f"{year}-02-02" for year in range(1925, 1951)))
        self.assertNoHoliday(f"{year}-02-02" for year in range(1951, 2050))
        self.assertNoHolidayName(name, range(1951, 2050))

    def test_easter_related(self):
        self.assertHoliday(
            # Niedziela Wielkanocna
            "2015-04-05",
            "2016-03-27",
            "2017-04-16",
            "2018-04-01",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            # Poniedziałek Wielkanocny
            "2015-04-06",
            "2016-03-28",
            "2017-04-17",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            # Zielone Świątki
            "2015-05-24",
            "2016-05-15",
            "2017-06-04",
            "2018-05-20",
            "2019-06-09",
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
            # Dzień Bożego Ciała
            "2015-06-04",
            "2016-05-26",
            "2017-06-15",
            "2018-05-31",
            "2019-06-20",
            "2020-06-11",
            "2021-06-03",
            "2022-06-16",
            "2023-06-08",
        )

    def test_national_day(self):
        name = "Święto Państwowe"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1950, 2050)))
        self.assertNoHoliday(f"{year}-05-01" for year in range(1925, 1950))
        self.assertNoHolidayName(name, range(1925, 1950))

    def test_national_day_of_third_of_may(self):
        name = "Święto Narodowe Trzeciego Maja"
        self.assertHolidayName(
            name, (f"{year}-05-03" for year in (*range(1925, 1951), *range(1990, 2050)))
        )
        self.assertNoHoliday(f"{year}-05-03" for year in range(1951, 1990))
        self.assertNoHolidayName(name, range(1951, 1990))

    def test_national_victory_and_freedom_day(self):
        name = "Narodowe Święto Zwycięstwa i Wolności"
        self.assertHolidayName(name, (f"{year}-05-09" for year in range(1946, 1951)))
        # Exclude Ascension day on 1929-05-09
        self.assertNoHoliday(
            f"{year}-05-09"
            for year in (*range(1925, 1929), *range(1930, 1946), *range(1951, 2050))
        )
        self.assertNoHolidayName(name, range(1925, 1946), range(1951, 2050))

    def test_ascension_day(self):
        name = "Wniebowstąpienie Pańskie"
        self.assertHolidayName(
            name,
            "1930-05-29",
            "1934-05-10",
            "1939-05-18",
            "1945-05-10",
            "1950-05-18",
        )
        self.assertHolidayName(name, range(1925, 1951))
        self.assertNoHolidayName(name, range(1951, 2050))

    def test_whit_monday(self):
        name = "Drugi dzień Zielonych Świątek"
        self.assertHolidayName(
            name,
            "1930-06-09",
            "1934-05-21",
            "1939-05-29",
            "1945-05-21",
            "1950-05-29",
        )
        self.assertHolidayName(name, range(1925, 1951))
        self.assertNoHolidayName(name, range(1951, 2050))

    def test_saints_peter_and_paul_day(self):
        name = "Uroczystość Świętych Apostołów Piotra i Pawła"
        self.assertHolidayName(name, (f"{year}-06-29" for year in range(1925, 1951)))
        self.assertNoHoliday(f"{year}-06-29" for year in range(1951, 2050))
        self.assertNoHolidayName(name, range(1951, 2050))

    def test_national_day_of_rebirth_of_poland(self):
        name = "Narodowe Święto Odrodzenia Polski"
        self.assertHolidayName(name, (f"{year}-07-22" for year in range(1945, 1990)))
        self.assertNoHoliday(f"{year}-07-22" for year in (*range(1925, 1945), *range(1990, 2050)))
        self.assertNoHolidayName(name, range(1925, 1945), range(1990, 2050))

    def test_assumption_day(self):
        name = "Wniebowzięcie Najświętszej Marii Panny"
        self.assertHolidayName(
            name, (f"{year}-08-15" for year in (*range(1925, 1961), *range(1989, 2050)))
        )
        self.assertNoHoliday(f"{year}-08-15" for year in range(1961, 1989))
        self.assertNoHolidayName(name, range(1961, 1989))

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Uroczystość Wszystkich Świętych", (f"{year}-11-01" for year in range(1925, 2050))
        )

    def test_national_independence_day(self):
        name = "Narodowe Święto Niepodległości"
        self.assertHolidayName(
            name, (f"{year}-11-11" for year in (*range(1937, 1945), *range(1989, 2050)))
        )
        self.assertNoHoliday(f"{year}-11-11" for year in (*range(1925, 1937), *range(1945, 1989)))
        self.assertNoHolidayName(name, range(1925, 1937), range(1945, 1989))

    def test_immaculate_conception_day(self):
        name = "Niepokalane Poczęcie Najświętszej Marii Panny"
        self.assertHolidayName(name, (f"{year}-12-08" for year in range(1925, 1951)))
        self.assertNoHoliday(f"{year}-12-08" for year in range(1951, 2050))
        self.assertNoHolidayName(name, range(1951, 2050))

    def test_christmas_eve(self):
        name = "Wigilia Bożego Narodzenia"
        self.assertHolidayName(name, (f"{year}-12-24" for year in range(2025, 2050)))
        self.assertNoHoliday(f"{year}-12-24" for year in range(1925, 2025))
        self.assertNoHolidayName(name, range(1925, 2025))

    def test_christmas(self):
        self.assertHolidayName(
            "Boże Narodzenie (pierwszy dzień)", (f"{year}-12-25" for year in range(1925, 2050))
        )
        self.assertHolidayName(
            "Boże Narodzenie (drugi dzień)", (f"{year}-12-26" for year in range(1925, 2050))
        )

    def test_2017(self):
        # http://www.officeholidays.com/countries/poland/2017.php
        self.assertHolidayDatesInYear(
            2017,
            "2017-01-01",
            "2017-01-06",
            "2017-04-16",
            "2017-04-17",
            "2017-05-01",
            "2017-05-03",
            "2017-06-04",
            "2017-06-15",
            "2017-08-15",
            "2017-11-01",
            "2017-11-11",
            "2017-12-25",
            "2017-12-26",
        )

    def test_2022(self):
        self.assertHolidayDatesInYear(
            2022,
            "2022-01-01",
            "2022-01-06",
            "2022-04-17",
            "2022-04-18",
            "2022-05-01",
            "2022-05-03",
            "2022-06-05",
            "2022-06-16",
            "2022-08-15",
            "2022-11-01",
            "2022-11-11",
            "2022-12-25",
            "2022-12-26",
        )

    def test_2025(self):
        self.assertHolidayDatesInYear(
            2025,
            "2025-01-01",
            "2025-01-06",
            "2025-04-20",
            "2025-04-21",
            "2025-05-01",
            "2025-05-03",
            "2025-06-08",
            "2025-06-19",
            "2025-08-15",
            "2025-11-01",
            "2025-11-11",
            "2025-12-24",
            "2025-12-25",
            "2025-12-26",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2018-01-01", "Nowy Rok"),
            ("2018-01-06", "Święto Trzech Króli"),
            ("2018-04-01", "Niedziela Wielkanocna"),
            ("2018-04-02", "Poniedziałek Wielkanocny"),
            ("2018-05-01", "Święto Państwowe"),
            ("2018-05-03", "Święto Narodowe Trzeciego Maja"),
            ("2018-05-20", "Zielone Świątki"),
            ("2018-05-31", "Dzień Bożego Ciała"),
            ("2018-08-15", "Wniebowzięcie Najświętszej Marii Panny"),
            ("2018-11-01", "Uroczystość Wszystkich Świętych"),
            ("2018-11-11", "Narodowe Święto Niepodległości"),
            ("2018-11-12", "Narodowe Święto Niepodległości - 100-lecie"),
            ("2018-12-25", "Boże Narodzenie (pierwszy dzień)"),
            ("2018-12-26", "Boże Narodzenie (drugi dzień)"),
        )

        self.assertLocalizedHolidays(
            ("2022-01-01", "Nowy Rok"),
            ("2022-01-06", "Święto Trzech Króli"),
            ("2022-04-17", "Niedziela Wielkanocna"),
            ("2022-04-18", "Poniedziałek Wielkanocny"),
            ("2022-05-01", "Święto Państwowe"),
            ("2022-05-03", "Święto Narodowe Trzeciego Maja"),
            ("2022-06-05", "Zielone Świątki"),
            ("2022-06-16", "Dzień Bożego Ciała"),
            ("2022-08-15", "Wniebowzięcie Najświętszej Marii Panny"),
            ("2022-11-01", "Uroczystość Wszystkich Świętych"),
            ("2022-11-11", "Narodowe Święto Niepodległości"),
            ("2022-12-25", "Boże Narodzenie (pierwszy dzień)"),
            ("2022-12-26", "Boże Narodzenie (drugi dzień)"),
        )

    def test_l10n_de(self):
        self.assertLocalizedHolidays(
            "de",
            ("2018-01-01", "Neujahr"),
            ("2018-01-06", "Heilige Drei Könige"),
            ("2018-04-01", "Ostersonntag"),
            ("2018-04-02", "Ostermontag"),
            ("2018-05-01", "Tag der Arbeit"),
            ("2018-05-03", "Nationalfeiertag am 3. Mai"),
            ("2018-05-20", "Pfingsten"),
            ("2018-05-31", "Fronleichnam"),
            ("2018-08-15", "Mariä Himmelfahrt"),
            ("2018-11-01", "Allerheiligen"),
            ("2018-11-11", "Nationalfeiertag der Unabhängigkeit"),
            ("2018-11-12", "Nationalfeiertag der Unabhängigkeit - 100. Jahrestag"),
            ("2018-12-25", "Erster Weihnachtstag"),
            ("2018-12-26", "Zweiter Weihnachtstag"),
        )
        self.assertLocalizedHolidays(
            "de",
            ("2022-01-01", "Neujahr"),
            ("2022-01-06", "Heilige Drei Könige"),
            ("2022-04-17", "Ostersonntag"),
            ("2022-04-18", "Ostermontag"),
            ("2022-05-01", "Tag der Arbeit"),
            ("2022-05-03", "Nationalfeiertag am 3. Mai"),
            ("2022-06-05", "Pfingsten"),
            ("2022-06-16", "Fronleichnam"),
            ("2022-08-15", "Mariä Himmelfahrt"),
            ("2022-11-01", "Allerheiligen"),
            ("2022-11-11", "Nationalfeiertag der Unabhängigkeit"),
            ("2022-12-25", "Erster Weihnachtstag"),
            ("2022-12-26", "Zweiter Weihnachtstag"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2018-01-01", "New Year's Day"),
            ("2018-01-06", "Epiphany"),
            ("2018-04-01", "Easter Sunday"),
            ("2018-04-02", "Easter Monday"),
            ("2018-05-01", "National Day"),
            ("2018-05-03", "National Day of the Third of May"),
            ("2018-05-20", "Pentecost"),
            ("2018-05-31", "Corpus Christi"),
            ("2018-08-15", "Assumption Day"),
            ("2018-11-01", "All Saints' Day"),
            ("2018-11-11", "National Independence Day"),
            ("2018-11-12", "National Independence Day - 100th anniversary"),
            ("2018-12-25", "Christmas Day"),
            ("2018-12-26", "Second Day of Christmas"),
        )
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "National Day"),
            ("2022-05-03", "National Day of the Third of May"),
            ("2022-06-05", "Pentecost"),
            ("2022-06-16", "Corpus Christi"),
            ("2022-08-15", "Assumption Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-11", "National Independence Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2018-01-01", "Новий рік"),
            ("2018-01-06", "Богоявлення"),
            ("2018-04-01", "Великдень"),
            ("2018-04-02", "Великодній понеділок"),
            ("2018-05-01", "Національне свято"),
            ("2018-05-03", "Національне свято Третього Травня"),
            ("2018-05-20", "День Святої Трійці"),
            ("2018-05-31", "Свято Тіла і Крові Христових"),
            ("2018-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2018-11-01", "День усіх святих"),
            ("2018-11-11", "День незалежності"),
            ("2018-11-12", "100-а річниця Дня незалежності"),
            ("2018-12-25", "Різдво Христове"),
            ("2018-12-26", "Другий день Різдва"),
        )
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-05-01", "Національне свято"),
            ("2022-05-03", "Національне свято Третього Травня"),
            ("2022-06-05", "День Святої Трійці"),
            ("2022-06-16", "Свято Тіла і Крові Христових"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-11", "День незалежності"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )
